/*
 * Created on 2006-2-24
 *
 */
package ambit.ui;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.lang.reflect.Method;

import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JToolBar;
import javax.swing.JPopupMenu.Separator;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledEditorKit;

import ambit.log.AmbitLogger;
import ambit.ui.actions.InsertSymbolAction;

/**
 * static finctions for icon loading and browser launch. 
 * @author Nina Jeliazkova nina@acad.bg
 * <b>Modified</b> 2006-2-24
 */
public class UITools {
	static AmbitLogger logger = new AmbitLogger(UITools.class);

    /**
     * 
     */
    protected UITools() {
        super();
    }
    public static ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = UITools.class.getClassLoader().getResource(path);

        if (imgURL != null) {
            return new ImageIcon(imgURL);
        } else {
            logger.error("Couldn't find file: " + path);
            return null;
        }
    }	
    /**
     * Launches a browser with specified url
     * @param url
     */
    public static void openURL(String url) {
        String osName = System.getProperty("os.name");
        try {
           if (osName.startsWith("Mac OS")) {
              Class fileMgr = Class.forName("com.apple.eio.FileManager");
              Method openURL = fileMgr.getDeclaredMethod("openURL",
                 new Class[] {String.class});
              openURL.invoke(null, new Object[] {url});
              }
           else if (osName.startsWith("Windows"))
              Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + url);
           else { //assume Unix or Linux
              String[] browsers = {
                 "firefox", "opera", "konqueror", "epiphany", "mozilla", "netscape" };
              String browser = null;
              for (int count = 0; count < browsers.length && browser == null; count++)
                 if (Runtime.getRuntime().exec(
                       new String[] {"which", browsers[count]}).waitFor() == 0)
                    browser = browsers[count];
              if (browser == null)
                 throw new Exception(browsers.toString());
              else
                 Runtime.getRuntime().exec(new String[] {browser, url});
              }
           }
        catch (Exception e) {
           JOptionPane.showMessageDialog(null, e.getMessage() + ":\n" + e.getLocalizedMessage());
           }
 
     }

	public static JMenu createEditMenu(Container mainFrame) {
		JMenu mainMenu = new JMenu("Edit");
		mainMenu.setMnemonic(KeyEvent.VK_E);
		JMenuItem menuItem;
		menuItem = new JMenuItem(new DefaultEditorKit.CutAction());
		menuItem.setText("Cut (Ctr-T)");
		menuItem.setMnemonic(KeyEvent.VK_T);
		mainMenu.add(menuItem);
		menuItem = new JMenuItem(new DefaultEditorKit.CopyAction());
		menuItem.setText("Copy (Ctrl-C)");
		menuItem.setMnemonic(KeyEvent.VK_C);
		mainMenu.add(menuItem);
		menuItem = new JMenuItem(new DefaultEditorKit.PasteAction());
		menuItem.setText("Paste (Ctrl-V)");
		menuItem.setMnemonic(KeyEvent.VK_P);
		mainMenu.add(menuItem);
		
		mainMenu.add(new Separator());
		menuItem = new JMenuItem(new InsertSymbolAction(mainFrame,"Insert symbol"));
		menuItem.setText("Insert symbol");
		menuItem.setMnemonic(KeyEvent.VK_P);
		mainMenu.add(menuItem);
		
		return mainMenu;
	}
	
	public static JMenu createStyleMenu() {
        JMenu menu = new JMenu("Style");

        Action action = new StyledEditorKit.BoldAction();
        action.putValue(Action.NAME, "Bold");
        menu.add(action);

        action = new StyledEditorKit.ItalicAction();
        action.putValue(Action.NAME, "Italic");
        menu.add(action);

        action = new StyledEditorKit.UnderlineAction();
        action.putValue(Action.NAME, "Underline");
        menu.add(action);

        menu.addSeparator();
        menu.add(new JMenuItem(new SubscriptAction()));		
        menu.add(new JMenuItem(new SuperscriptAction()));

        menu.addSeparator();
        menu.add(new StyledEditorKit.FontSizeAction("12", 12));
        menu.add(new StyledEditorKit.FontSizeAction("14", 14));
        menu.add(new StyledEditorKit.FontSizeAction("18", 18));

        menu.addSeparator();

        menu.add(new StyledEditorKit.FontFamilyAction("Serif",
                                                      "Serif"));
        menu.add(new StyledEditorKit.FontFamilyAction("SansSerif",
                                                      "SansSerif"));

        menu.addSeparator();

        menu.add(new StyledEditorKit.ForegroundAction("Red",
                                                      Color.red));
        menu.add(new StyledEditorKit.ForegroundAction("Green",
                                                      Color.green));
        menu.add(new StyledEditorKit.ForegroundAction("Blue",
                                                      Color.blue));
        menu.add(new StyledEditorKit.ForegroundAction("Black",
                                                      Color.black));

        return menu;
    }
	
	public static JComponent createToolbar(int width, int height) {
	    JToolBar toolBar = new JToolBar(JToolBar.HORIZONTAL  ) ;
        toolBar.setPreferredSize(new Dimension(width,  height));
        toolBar.setFloatable(false);
        return toolBar;
	}	
}



class SubscriptAction extends StyledEditorKit.StyledTextAction{			
	public SubscriptAction(){			
		super("<html>x<sub>2</sub></html>"); //StyleConstants.Subscript.toString());
		putValue(SHORT_DESCRIPTION,"Subscript");
	}		
	public void actionPerformed(ActionEvent ae){			
		JEditorPane editor = getEditor(ae);			
		if (editor != null) {				
			StyledEditorKit kit = getStyledEditorKit(editor);				
			MutableAttributeSet attr = kit.getInputAttributes();				
			boolean subscript = (StyleConstants.isSubscript(attr)) ? false : true;				
			SimpleAttributeSet sas = new SimpleAttributeSet();				
			StyleConstants.setSubscript(sas, subscript);				
			setCharacterAttributes(editor, sas, false);			
		}		
	}	
}		

class SuperscriptAction extends StyledEditorKit.StyledTextAction{			
	public SuperscriptAction(){			
		super("<html>x<sup>2</sup></html>"); //StyleConstants.Superscript.toString());
		putValue(SHORT_DESCRIPTION,"Superscript");
	}		
	public void actionPerformed(ActionEvent ae){			
		JEditorPane editor = getEditor(ae);			
		if (editor != null) {				
			StyledEditorKit kit = getStyledEditorKit(editor);				
			MutableAttributeSet attr = kit.getInputAttributes();				
			boolean superscript = (StyleConstants.isSuperscript(attr)) ? false : true;				
			SimpleAttributeSet sas = new SimpleAttributeSet();				
			StyleConstants.setSuperscript(sas, superscript);				
			setCharacterAttributes(editor, sas, false);			
		}					
	}	
}
