/*
Copyright Ideaconsult (C) 2005-2007 
Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

package chemCast.data;

import javax.swing.JOptionPane;

import org.openscience.cdk.CDKConstants;
import org.openscience.cdk.interfaces.IChemObject;
import org.openscience.cdk.io.ReaderEvent;
import org.openscience.cdk.io.iterator.IIteratingChemObjectReader;
import org.openscience.cdk.io.listener.IReaderListener;
import org.openscience.cdk.io.setting.IOSetting;

import sun.security.action.GetLongAction;

import chemCast.application.StatusBar;

import ambit.data.molecule.IAtomContainersList;
import ambit.data.molecule.PropertyImporter;
import ambit.data.molecule.PropertyTranslator;
import ambit.exceptions.AmbitException;
import ambit.io.MolPropertiesIOSetting;
import ambit.io.batch.BatchProcessingException;
import ambit.io.batch.DefaultBatchConfig;
import ambit.io.batch.DefaultBatchProcessing;
import ambit.misc.AmbitCONSTANTS;
import ambit.processors.DefaultAmbitProcessor;
import ambit.processors.IAmbitProcessor;
import ambit.processors.IdentifiersProcessor;
import ambit.processors.ProcessorsChain;
import ambit.processors.structure.SmilesGeneratorProcessor;
import ambit.ui.data.PropertiesPanel;

public class BatchDescriptorImporter extends DefaultBatchProcessing {
    protected transient IdentifiersProcessor identifiersProcessor;
    protected transient SmilesGeneratorProcessor smilesGenerator;
    protected transient IAmbitProcessor transferProcessor;
    /**
     * 
     */
    private static final long serialVersionUID = 6772214233639036377L;

    public BatchDescriptorImporter(IIteratingChemObjectReader reader,
            IAtomContainersList destination, PropertyTranslator properties, Object lookupProperty) throws BatchProcessingException {
        super(reader,
                new PropertyImporter(destination,properties,lookupProperty),
                createProcessor(),new DefaultBatchConfig());
        //reader.addChemObjectIOListener(getReaderListener());
        //identifiersProcessor = (IdentifiersProcessor) ((ProcessorsChain)getProcessor()).get(0);
        //smilesGenerator = (SmilesGeneratorProcessor) ((ProcessorsChain)getProcessor()).get(1);
        smilesGenerator = (SmilesGeneratorProcessor) ((ProcessorsChain)getProcessor()).get(0);
        //transferProcessor = ((ProcessorsChain)getProcessor()).get(2);
        if (properties == null) {
            getImporter().setProperties(new PropertyTranslator());
        }
    }
    public static IAmbitProcessor createProcessor() {
        ProcessorsChain pc = new ProcessorsChain();
        //pc.add(new IdentifiersProcessor());
        SmilesGeneratorProcessor gen = new SmilesGeneratorProcessor();
        gen.setEnabled(false);
        pc.add(gen);
        /*
        IAmbitProcessor p = new DefaultAmbitProcessor() {
            @Override
            public Object process(Object object) throws AmbitException {
                ((IChemObject) object).setProperty("SMILES",
                        ((IChemObject) object).getProperty(AmbitCONSTANTS.UNIQUE_SMILES));
                return object;
            }
        };
        p.setEnabled(false);
        pc.add(p);
        */
        return pc;
    }
    public void updateProperties(PropertyTranslator newProperties,Object lookupProperty) {
        getImporter().getProperties().add(newProperties);
        getImporter().setLookupProperty(lookupProperty);
        if (identifiersProcessor != null)
        identifiersProcessor.addIdentifiers(getImporter().getProperties().getIdentifiers());
        if ("SMILES".equals(lookupProperty)) {
            smilesGenerator.setEnabled(true);
            //transferProcessor.setEnabled(true);
            getImporter().setLookupProperty(AmbitCONSTANTS.UNIQUE_SMILES);
        }
    }
    protected PropertyImporter getImporter() {
        return (PropertyImporter) getWriter();
    }
    protected IReaderListener getReaderListener() {
        return new IReaderListener() {
          public void frameRead(ReaderEvent e) {
          }
          public void processIOSettingQuestion(IOSetting setting) {
              if (setting instanceof MolPropertiesIOSetting) {
                  if (getImporter().getProperties().isEmpty()) {
                      
                      if (JOptionPane.showConfirmDialog(null, 
                          new PropertiesPanel(((MolPropertiesIOSetting)setting).getProperties(),
                                  new String[] {PropertyTranslator.type_identifiers,PropertyTranslator.type_descriptors},
                                  null)
                      ,"Select descriptors to be imported",JOptionPane.OK_CANCEL_OPTION,JOptionPane.PLAIN_MESSAGE,null
                      ) == JOptionPane.OK_OPTION) {
                          updateProperties(((MolPropertiesIOSetting)setting).getProperties(),"");
                      }
                  }
              }  
          }
        };
    }
}
