/* AtomEnvironmentSimilarity.java
 * Author: Nina Jeliazkova
 * Date: Nov 18, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.qsar.model.QSARModelException;

import ambit.data.descriptors.AtomEnvironmentList;
import ambit.data.molecule.IAtomContainersList;
import ambit.exceptions.AmbitException;
import ambit.misc.AmbitCONSTANTS;
import ambit.processors.structure.AtomEnvironmentGenerator;
import ambit.similarity.DefaultSimilarityProcessor;
import ambit.similarity.DoubleValuesHistogram;
import ambit.stats.datastructures.Bin;
import chemCast.application.StatusBar;
import chemCast.weka.ISubsetsModel;
/**
 * 
 * Calculates set of consensus atom environments and calculates Hellinger distance between query compound and consensus a.e.  
 * @author Nina Jeliazkova nina@acad.bg
 * <b>Modified</b> Feb 4, 2007
 */
public class AtomEnvironmentSimilarity extends DefaultSimilarityProcessor implements  ISubsetsModel {
    /**
     * 
     */
    private static final long serialVersionUID = 7088647920846786278L;
    protected Subsets datasets = null;
    protected AtomEnvironmentGenerator generator;
    protected AtomEnvironmentList consensusAE;
    
    
    public AtomEnvironmentSimilarity() {
        super();
        setPredicting(false);
//        aeComparison = new AtomEnvironmentDistanceType(0);
        generator = new AtomEnvironmentGenerator();
        generator.setUseHydrogens(false);
        consensusAE = new AtomEnvironmentList();
    }

    public Object getParameter(Object parameter) {
        // TODO Auto-generated method stub
        return null;
    }

    public void setParameter(Object parameter, Object value) {
        // TODO Auto-generated method stub

    }
    /* (non-Javadoc)
     * @see chemCast.weka.ISubsetsModel#getDatasets()
     */
    public Subsets getDatasets() {
        return datasets;
    }

    /* (non-Javadoc)
     * @see chemCast.weka.ISubsetsModel#setDatasets(chemCast.similarity.data.Subsets)
     */
    public void setDatasets(Subsets datasets) {
        this.datasets = datasets;
    }
    protected AtomEnvironmentList getAtomEnvironments(IAtomContainersList list, int index) {
        Object ae = list.getProperty(index, AmbitCONSTANTS.AtomEnvironment);
        if ((ae != null) && (!"".equals(ae))) return (AtomEnvironmentList) ae;
        try {
            IAtomContainer ac = list.getAtomContainer(index);
            generator.process(ac);
            ae = ac.getProperty(AmbitCONSTANTS.AtomEnvironment);
            if (ae instanceof AtomEnvironmentList) {
                list.setProperty(index, AmbitCONSTANTS.AtomEnvironment,ae);
                return (AtomEnvironmentList)ae;    
            } else {
                list.setProperty(index, AmbitCONSTANTS.AtomEnvironment,null);
                return null;
            }
        } catch (AmbitException x) {
            x.printStackTrace();
            list.setProperty(index, AmbitCONSTANTS.AtomEnvironment, null);
            return null;
        }
        

    }
    public void build() throws QSARModelException {
        consensusAE.clear();
        //histogram.clear();
        trainingPrefix = datasets.toString();
        datasets.firePropertyChange(StatusBar.Message_Status,null,"Build phase: " + toString() + " " + datasets.toString());

        for (int s=0; s < datasets.size();s++)
            if (datasets.get(s).isEnabled()) 
            try {
                IAtomContainersList list = datasets.get(s).getData();
                if (list == null) continue;
                for (int i=0; i < list.getAtomContainerCount();i++) 
                    try {
                        datasets.firePropertyChange(StatusBar.Message_Progress,null,new Double(100*i/list.getAtomContainerCount()));
                        AtomEnvironmentList ae = getAtomEnvironments(list, i);
                        consensusAE.addAll(ae);
                    } catch (Exception x) {
                        x.printStackTrace();
                    }
                    
            } catch (Exception x) {
                datasets.firePropertyChange(StatusBar.Message_Error,null,x);
            }
 
       setPredicting(true);     
       setDatasets(null);

    }
    public void predict() throws QSARModelException {
        //histogram.clear();
        if (modelStatistics != null) {
        	modelStatistics.clear(datasets.toString());
        	modelStatistics.setTitle(getSimilarityProperty());
        }
        datasets.firePropertyChange(StatusBar.Message_Status,null,"Prediction phase: " + toString() + " " + datasets.toString());
        String sp = trainingPrefix + "." + getSimilarityProperty();
        for (int s=0; s < datasets.size();s++)
            if (datasets.get(s).isEnabled()) 
            try {
                IAtomContainersList list = datasets.get(s).getData();
                if (list == null) continue;
                for (int i=0; i < list.getAtomContainerCount();i++) 
                    try {
                        datasets.firePropertyChange(StatusBar.Message_Progress,null,new Double(100*i/list.getAtomContainerCount()));
                        AtomEnvironmentList ae = getAtomEnvironments(list, i);
                        Double distance = null;
                        if (ae != null) {
                            distance = new Double(ae.hellinger(consensusAE)/2.0);
                            list.setProperty(i, sp,distance);
                            //histogram.update(distance);
                            if (modelStatistics == null) modelStatistics = new DoubleValuesHistogram<Bin>(getSimilarityProperty());
                            modelStatistics.addValue(datasets.toString(),datasets.get(s).toString(), distance);
                            
                        } else
                            list.setProperty(i, sp, null);
                        
                    } catch (Exception x) {
                        datasets.firePropertyChange(StatusBar.Message_Error,null,x);
                        list.setProperty(i, sp, null);
                    }
                    
            } catch (Exception x) {
                x.printStackTrace();
            }
         setDatasets(null);
        
    }
    protected AtomEnvironmentList getAE(Object object1) throws Exception {
        if (object1 instanceof AtomEnvironmentList) return (AtomEnvironmentList)object1;
        else if (object1 instanceof IAtomContainer) { 
            generator.process((IAtomContainer) object1);
            return (AtomEnvironmentList) ((IAtomContainer) object1).getProperty(AmbitCONSTANTS.AtomEnvironment);
        } else return null;
    }
    public double predict(Object object1, Object object2) throws AmbitException {
        try {
            AtomEnvironmentList list1 = getAE(object1);
            AtomEnvironmentList list2 = getAE(object2);
            return list1.hellinger(list2)/2.0;
        } catch (Exception x) {
            throw new AmbitException(x);
        }
        
    }
    public float predict(IAtomContainersList object1, int index1, IAtomContainersList object2, int index2) throws AmbitException {
        try {
            Object list1 = object1.getProperty(index1, AmbitCONSTANTS.AtomEnvironment);
            if (list1==null) list1 = getAE(object1.getAtomContainer(index1));
            Object list2 = object2.getProperty(index2, AmbitCONSTANTS.AtomEnvironment);
            if (list2==null) list2 = getAE(object2.getAtomContainer(index2));
            return (float) ((AtomEnvironmentList)list1).hellinger((AtomEnvironmentList)list2)/ 2.0f;
        } catch (Exception x) {
            throw new AmbitException(x);
        }
    }
    @Override
    public void close() {
        super.close();
    }
    @Override
    public String toString() {
        return "Similarity by Atom environments";
    }
    public String[] getProperties() {
        return new String[] {getTrainingPrefix() + "." + getSimilarityProperty()};
    }

    /*
     *  public String[] getProperties() {
        if (doesPrediction)
            return new String[]{
            getTrainingPrefix() + "." + getSimilarityProperty(),
            getTrainingPrefix() + "." + getSimilarityProperty() + "." + getPredictedProperty()};
        else
            if (convertor.isEmptyClassProperty())
                return new String[]{
                    getTrainingPrefix() + "." + getSimilarityProperty()
                    };          

            else {
                String[] p =  new String[classValues.length+1];
                p[0] = getTrainingPrefix() + "." + getSimilarityProperty() + "." + getClassificationProperty();
                for (int i=1; i < p.length;i++)  p[i] = getTrainingPrefix() + "." + getSimilarityProperty() + "." + classValues[i-1];
                return p;
            }
    }
     */
}
