/* DatasetDef.java
 * Author: Nina Jeliazkova
 * Date: Sep 24, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.awt.Color;
import java.awt.Dimension;
import java.io.File;
import java.io.FileInputStream;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import org.openscience.cdk.DefaultChemObjectBuilder;
import org.openscience.cdk.io.iterator.IIteratingChemObjectReader;
import org.openscience.cdk.io.listener.IReaderListener;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import ambit.data.IAmbitEditor;
import ambit.data.literature.ReferenceFactory;
import ambit.data.molecule.IAtomContainersList;
import ambit.data.molecule.MoleculesIterator;
import ambit.data.molecule.PropertyTranslator;
import ambit.io.FileInputState;
import ambit.ui.data.PropertiesPanel;
import chemCast.data.MoleculesFile;
import chemCast.exception.SubsetException;

public class DatasetDef extends Subset {
    public static String p_source = "trainingset";
    public static String p_query = "testset";    
    public static String p_filesource = "file";
    public static String p_memorysource = "New dataset";
    public static String p_template = "template";
    public static String p_result = "result";
    public static String p_identifiers = "identifiers";
    public static String p_identifier = "identifier";
    public static String p_descriptors = "descriptors";
    public static String p_descriptor = "descriptor";
    protected String datasetSource = p_filesource;
    
    protected String datasetValue = "";
    protected String datasetTemplate = "";
    protected String datasetResultField = "";

    protected PropertyTranslator properties;
    
    public DatasetDef() throws SubsetException {
        this("default.sdf","template.xml","result");
    }
    public DatasetDef(Element element) throws Exception {
        this();
        setXMLElement(element);
    }
    /**
     * 
     * @param filename
     * @param templateName
     * @param resultField
     */
    public DatasetDef(String filename, String templateName, String resultField) throws SubsetException {
        super();
        setColor(Color.red);
        setDatasetSource(p_filesource);
        setDatasetValue(filename);
        setDatasetTemplate(templateName);
        setDatasetResultField(resultField);
        //properties = new MolProperties();
        properties = new PropertyTranslator(new String[] {p_identifiers,p_descriptors});
    }
    @Override
    public Element getXMLElement(Document document) {
        Element e = document.createElement(p_source);
        e.setAttribute(datasetSource, getDatasetValue());
        
        Element e1 = document.createElement(p_template);
        e1.setAttribute(p_filesource, getDatasetTemplate());

        Element e2 = document.createElement(p_result);
        e2.setAttribute(p_name, getDatasetResultField());
        
        e.appendChild(e1);
        e.appendChild(e2);
        
        e.appendChild(getXMLElement(document, properties.getProperties(p_identifiers), p_identifiers, p_identifier));
        e.appendChild(getXMLElement(document, properties.getProperties(p_descriptors), p_descriptors, p_descriptor));        
        /*
        e.appendChild(getXMLElement(document, properties.getIdentifiers(), p_identifiers, p_identifier));
        e.appendChild(getXMLElement(document, properties.getDescriptors(), p_descriptors, p_descriptor));
        */
        return e;
    }
    public Element getXMLElement(Document document, Hashtable hashtable, String tableId, String itemId) {
        Element e = document.createElement(tableId);
        Enumeration<?> i = hashtable.keys();
        while (i.hasMoreElements()) {
            Object key = i.nextElement();
            Object value = hashtable.get(key);
            Element e1 = document.createElement(itemId);
            e1.setAttribute(p_tag, key.toString());
            e1.setAttribute(p_name, value.toString());
            e.appendChild(e1);
        }
        return e;
    }
    
    @Override
    public void setXMLElement(Element element) throws Exception {
        if (element.getNodeName().equals(p_source)) {
            String file = element.getAttribute(p_filesource);
            if (file != null) {
                setDatasetValue(file);
                setDatasetSource(p_filesource);
            }
            
            setXMLElement(element, properties.getProperties(p_identifiers), p_identifiers,p_identifier);
            setXMLElement(element, properties.getProperties(p_descriptors), p_descriptors,p_descriptor);            
            /*
            setXMLElement(element, properties.getIdentifiers(), p_identifiers,p_identifier);
            setXMLElement(element, properties.getDescriptors(), p_descriptors,p_descriptor);
            */
            //element.appendChild(getSource().setXMLElement(document));
            NodeList nodes = element.getElementsByTagName(p_template);
            if (nodes != null) 
                for (int i=0; i < nodes.getLength(); i++) {
                    setDatasetTemplate(((Element)nodes.item(i)).getAttribute(p_filesource));
                    break;
                }     
            nodes = element.getElementsByTagName(p_result);
            if (nodes != null) 
                for (int i=0; i < nodes.getLength(); i++) {
                    setDatasetResultField(((Element)nodes.item(i)).getAttribute(p_name));
                    break;
                }                     
        } else throw new Exception("Not an element of type "+p_source);


    }
    public void setXMLElement(Element element,Hashtable hashtable, String tableId, String itemId) {
        NodeList ids = element.getElementsByTagName(tableId);
        for (int k=0; k < ids.getLength(); k++) {
        
            NodeList nodes = ((Element) ids.item(k)).getElementsByTagName(itemId);
            for (int i=0; i < nodes.getLength(); i++) {
                Element identifier = ((Element)nodes.item(i));
                try {
                    hashtable.put(identifier.getAttribute(p_tag),
                        identifier.getAttribute(p_name));
                } catch (Exception x) {
                    x.printStackTrace();
                }
            }     
        }
    }
    public int compareTo(Object o) {
        // TODO Auto-generated method stub
        return 0;
    }
    public synchronized String getDatasetResultField() {
        return datasetResultField;
    }
    public synchronized void setDatasetResultField(String datasetResultField) {
        this.datasetResultField = datasetResultField;
    }
    public synchronized String getDatasetSource() {
        return datasetSource;
    }
    public synchronized void setDatasetSource(String datasetSource) {
        this.datasetSource = datasetSource;
    }
    public synchronized String getDatasetTemplate() {
        return datasetTemplate;
    }
    public synchronized void setDatasetTemplate(String datasetTemplate) {
        this.datasetTemplate = datasetTemplate;
    }
    public synchronized String getDatasetValue() {
        return datasetValue;
    }
    public synchronized void setDatasetValue(String datasetValue) {
        this.datasetValue = datasetValue;
    }
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof DatasetDef) {
            DatasetDef e = (DatasetDef) obj;
            return getDatasetValue().equals(getDatasetValue()) &&
            getDatasetResultField().equals(e.getDatasetResultField()) &&
            getDatasetSource().equals(e.getDatasetSource()) &&
            getDatasetTemplate().equals(e.getDatasetTemplate());
        } else return false;

    }
    @Override
    public String toString() {
        return getDatasetSource() + " " + getDatasetValue();
    }
    public synchronized PropertyTranslator getProperties() {
        return properties;
    }
    public synchronized void setProperties(PropertyTranslator properties) {
        this.properties = properties;
        try {
	        IAtomContainersList list = parentDataset;
	        if (list != null)
	        	list.addAvailableProperties(properties);
        } catch (Exception x) {
        	x.printStackTrace();
        }
    }
    public synchronized void addProperties(PropertyTranslator properties) {
        this.properties.add(properties);
        try {
            IAtomContainersList list = parentDataset;
            if (list != null)
                list.addAvailableProperties(properties);
        } catch (Exception x) {
            x.printStackTrace();
        }
    }

    protected String extractResultField(PropertyTranslator properties) {
        String r = "";
        Hashtable h = properties.getProperties(PropertyTranslator.type_results);
        if (h != null) {
            Enumeration e = h.keys();
            while (e.hasMoreElements()) {
                r = e.nextElement().toString();
                break;
            }
        }
        return r;
    }
    //protected IAtomContainersList createAtomContainersList(File file, IReaderListener listener, Hashtable<String,String[]> options) throws SubsetException {
    protected IAtomContainersList createAtomContainersList(File file, IReaderListener listener, PropertyTranslator options) throws SubsetException {
    	IAtomContainersList list = null;
        try {
            MoleculesFile mf = new MoleculesFile(file, DefaultChemObjectBuilder.getInstance(),listener);
            setName(file.getName());
            if (properties.getProperties().size()>0) 
                JOptionPane.showConfirmDialog(null,
                    new PropertiesPanel(properties,
                            new String[] 
                            {PropertyTranslator.type_identifiers,
                            PropertyTranslator.type_descriptors,
                            PropertyTranslator.type_results},
                            options),
                    "",JOptionPane.OK_CANCEL_OPTION,JOptionPane.PLAIN_MESSAGE,null);
            		             
            mf.addAvailableProperties(properties);
            if (getDatasetResultField().equals(""))
                setDatasetResultField(extractResultField(properties));
            //mf.bufferedProperties(getChemCastData().getProperties());
            list = mf;
        } catch (Exception x) {
            try {
                IIteratingChemObjectReader reader = FileInputState.getReader(new FileInputStream(file), file.getName());
                if (listener != null)
                	reader.addChemObjectIOListener(listener);
                list = new MoleculesIterator();

                while (reader.hasNext()) { 
                    list.add(reader.next());
                }   
                if (listener != null)
                	reader.removeChemObjectIOListener(listener);
                reader.close();
                if (properties.getProperties().size()>0) 
                JOptionPane.showConfirmDialog(null,
                		new PropertiesPanel(properties,
                        new String[] {PropertyTranslator.type_identifiers,
                                PropertyTranslator.type_descriptors,
                                PropertyTranslator.type_results},
                                options)
                		,"",JOptionPane.OK_CANCEL_OPTION,JOptionPane.PLAIN_MESSAGE,null);
//                		== JOptionPane.OK_OPTION); 
               	list.addAvailableProperties(properties);
                 if (getDatasetResultField().equals(""))
                        setDatasetResultField(extractResultField(properties));

            } catch (Exception xx) {
                throw new SubsetException(xx);
            }
        }
        return list;
    }
    
    
    //public IAtomContainersList createData(IReaderListener listener,Hashtable<String,String[]> options) throws SubsetException {
    public IAtomContainersList createData(IReaderListener listener,PropertyTranslator options) throws SubsetException {
    	if (data == null)
    		if (p_filesource.equals(getDatasetSource()))
    			setData(createAtomContainersList(new File(getDatasetValue()),listener,options));
    		else if (p_memorysource.equals(getDatasetSource()))
                setData(new MoleculesIterator());
            else throw new SubsetException("Not a file source "+ getDatasetSource());	
    	return data;
    }
    @Override
    public IAtomContainersList getData() throws SubsetException {
    	return data;
    }
    @Override
    public void setData(IAtomContainersList data) throws SubsetException{
    	this.data = data;
        setParentDataset(data);
    }
    
	public void setParentDataset(IAtomContainersList list) throws SubsetException {
		this.parentDataset = list;
		firePropertyChange(p_group, null, this);
	}

    public ChemCastDescriptorsList getDescriptorsList() {
        ChemCastDescriptorsList d = new ChemCastDescriptorsList();
        Enumeration e = properties.getDescriptorProperties().keys();
        while (e.hasMoreElements()) {
            Object key = e.nextElement();
            ChemCastDescriptor descriptor = new ChemCastDescriptor(
                    properties.getDescriptorProperties().get(key).toString(),ReferenceFactory.createEmptyReference());
            descriptor.setEnabled(true);
            d.add(descriptor);
        }   
        return d;
    }
    public IAmbitEditor getEditor() {
        return new DatasetDefEditor("Dataset details",this);
    }

}

class DatasetDefEditor extends SubsetEditor implements IAmbitEditor {

protected JTextField resultField;

public DatasetDefEditor(String title,DatasetDef object) {
    super(title,object);
    nameField.setText(object.getDatasetValue());
    //((TitledBorder)nameField.getBorder()).setTitle(object.getDatasetSource());
    resultField = new JTextField(object.getDatasetResultField());
    resultField.addActionListener(this);
    resultField.addFocusListener(this);
    resultField.setToolTipText("This field contains experimental values of the specified activity");
    resultField.setPreferredSize(new Dimension(100,24));
    resultField.setMaximumSize(new Dimension(Integer.MAX_VALUE,24));
    add(new JLabel("Endpoint"));
    add(resultField);    
    
}
@Override
public void setEditable(boolean editable) {
    super.setEditable(editable);
    resultField.setEditable(editable);
}
}    