/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.test.data;

import java.io.File;
import java.util.Enumeration;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import junit.framework.TestCase;

import org.openscience.cdk.qsar.DescriptorValue;
import org.openscience.cdk.qsar.IMolecularDescriptor;
import org.openscience.cdk.qsar.descriptors.molecular.XLogPDescriptor;
import org.openscience.cdk.qsar.result.DoubleResult;
import org.openscience.cdk.templates.MoleculeFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import weka.attributeSelection.AttributeEvaluator;
import weka.attributeSelection.ReliefFAttributeEval;
import weka.core.Attribute;
import weka.core.Instances;
import ambit.data.literature.ReferenceFactory;
import chemCast.application.ChemCASTModule;
import chemCast.similarity.data.ChemCastDescriptor;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.ChemCastObjectFactory;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.Endpoint;
import chemCast.similarity.data.Endpoints;
import chemCast.similarity.data.IChemCastObject;
import chemCast.similarity.data.Subsets;
import chemCast.weka.DescriptorSelection;

public class ChemCastDescriptorTest extends TestCase {

	public static void main(String[] args) {
		junit.textui.TestRunner.run(ChemCastDescriptorTest.class);
	}

	public ChemCastDescriptorTest(String arg0) {
		super(arg0);
	}

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	public void test() {
		String dclass = "org.openscience.cdk.qsar.descriptors.molecular.XLogPDescriptor";
		ChemCastDescriptor d = new ChemCastDescriptor("logP",ReferenceFactory.createEmptyReference());
        d.setRequired3D(true);
		d.setRemark("Membrane penetration");
		d.setDescriptorClass(dclass);
        assertTrue(d.isEnabled());
        try {
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document doc = builder.newDocument();
            Element e = d.getXMLElement(doc);
            assertEquals("logP", e.getAttribute(IChemCastObject.p_name));
            assertEquals(dclass, e.getAttribute(IChemCastObject.p_class));
            assertEquals("Membrane penetration", e.getAttribute(ChemCastDescriptor.p_note));
            assertEquals("3D", e.getAttribute(ChemCastDescriptor.p_type));
            doc.appendChild(e);
            ChemCastObjectFactory.writeXmlFile(doc, "test.xml");
            ChemCastDescriptor d1 = new ChemCastDescriptor("",ReferenceFactory.createEmptyReference());
            d1.setXMLElement(e);
            assertTrue(d1.isEnabled());
            assertEquals("logP", d1.getName());
            assertEquals(dclass, d1.getDescriptorClass());
            assertEquals("Membrane penetration", d1.getRemark());
            assertTrue( d1.isRequired3D());         
            IMolecularDescriptor calculator = d1.getCalculator();
            assertTrue(calculator instanceof XLogPDescriptor);
            DescriptorValue value = d1.calculate(MoleculeFactory.makeBenzene());
            assertTrue(value.getValue() instanceof DoubleResult);
            
        } catch (Exception e) {
            e.printStackTrace();
            fail();
        }

	}
	protected void clean(ChemCastDescriptorsList d) {
		for (int i=0; i < d.getList().size();i++)
			d.get(i).setWeight(-9999);
	}
    /*
	public void testDescriptorSelection() {
		 Endpoints e = ChemCASTModule.readEndpoints();
		 //ChemCastDescriptorsList d = ChemCASTModule.readDescriptors();
		 for (int i=0; i < e.size(); i++) {
			 Endpoint endpoint = e.get(i);
             DatasetDef dataset = endpoint.getTrainingData();
             File f = new File(dataset.getDatasetValue());
             if (!f.exists()) continue;
             ChemCastDescriptorsList d = dataset.getDescriptorsList();
			 for (int s=0; s < endpoint.getSubsets().size();s++) {
				 Subsets subsets = endpoint.getSubsets().get(s);
				 
				 
				 Subsets newSubsets = new Subsets();
				 try {
					 //dataset.getData();
					 newSubsets.createSubsets(dataset, subsets);
                     for (int n=0; n < newSubsets.size();n++) {
                         newSubsets.get(n).setEnabled(true);
                         if (newSubsets.get(n).getData().getAtomContainerCount()==0) {
                             System.out.println("NO DATA "+newSubsets.get(n));
                             newSubsets.get(n).setEnabled(false);
                         }
                     }
					 dataset.setEnabled(false);
					 
					 
					 clean(d);
					 if (DescriptorSelection.selectBestDescriptorsForSubsets(d, newSubsets)) {
    					 System.out.println();
    					 System.out.println(dataset);
    					 System.out.println(subsets.toString());
    					 for (int j=0; j < d.getList().size();j++) 
    					 	if (d.get(j).isEnabled()){
                                
    							 assertTrue(d.get(j).getWeight()!= -9999);
    					 	}
                     }
				 } catch (Exception x) {
					 x.printStackTrace();
					 fail();
				 }
                 
			 }
             try {
                 Subsets newSubsets = new Subsets();
                 newSubsets.add(dataset);
                 for (int n=0; n < newSubsets.size();n++)
                     newSubsets.get(n).setEnabled(false);
                 
                 dataset.setEnabled(true);
                 if (DescriptorSelection.selectBestDescriptorsForDataset(d,dataset.getDatasetResultField(), newSubsets)) {
                     System.out.println();
                     System.out.println(dataset.getDatasetResultField());
                     for (int j=0; j < d.getList().size();j++) 
                            if (d.get(j).isEnabled()){
                                 System.out.print(d.get(j).getName());
                                System.out.print('\t');
                                 System.out.println(d.get(j).getWeight());
                                 assertTrue(d.get(j).getWeight()!= -9999);
                            }
                 }
                 //Descriptors + fingerprints
                 Instances fpInstances = DescriptorSelection.properties2Instances(d,dataset.getDatasetResultField(), true,newSubsets);
                 AttributeEvaluator evaluator = new ReliefFAttributeEval();
                 evaluator.buildEvaluator(fpInstances);
                 
                 Enumeration en =  fpInstances.enumerateAttributes();
                 while (en.hasMoreElements()) {
                     Attribute att = (Attribute)en.nextElement();
                     double w = evaluator.evaluateAttribute((att).index());
                     if (w >0.5) {
                         System.out.print(att);
                         System.out.print('\t');
                         System.out.print(w);
                         System.out.println();
                     } //else fpInstances.deleteAttributeAt(att.index());
                 }
                 
             } catch (Exception x) {
                 
             }

		 }
	}
    */
}


