/* SimilarityDefTest.java
 * Author: Nina Jeliazkova
 * Date: Sep 23, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.test.data;

import java.util.ArrayList;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import junit.framework.TestCase;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import ambit.data.literature.ReferenceFactory;
import chemCast.similarity.data.ChemCastDescriptor;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.ChemCastObjectFactory;
import chemCast.similarity.data.IChemCastObject;
import chemCast.similarity.data.SimilarityDef;
import chemCast.similarity.data.SimilarityDefs;
import chemCast.similarity.data.SimilarityModel;

public class SimilarityDefTest extends TestCase {
    public void test() {
        SimilarityDef d = new SimilarityDef();
        assertEquals("ambit.similarity.FingerprintSimilarity", d.getClassName());
        try {
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document doc = builder.newDocument();
            Element e = d.getXMLElement(doc);
            doc.appendChild(e);
            
            SimilarityDef d1 = new SimilarityDef((Element)doc.getChildNodes().item(0));
            assertEquals(d,d1);
        } catch (Exception x) {
            fail();
        }
        
    }
    public void testlistToXML() {
        SimilarityDef s = new SimilarityDef("MyName1","MyType","","x");
        s.getParameters().put("Test", "True");
        ChemCastDescriptorsList d = new ChemCastDescriptorsList();
        d.add(new ChemCastDescriptor("descriptor1", ReferenceFactory.createEmptyReference()));
        d.add(new ChemCastDescriptor("descriptor2",ReferenceFactory.createEmptyReference()));
        s.setDescriptors(d);
        SimilarityDef s1 = new SimilarityDef("MyName2","MyType","MOA","y");
        try {
            ArrayList<IChemCastObject> l = new ArrayList<IChemCastObject>();
            l.add(s); l.add(s1);
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document doc = builder.newDocument();
            Element e = ChemCastObjectFactory.getXMLElement(l, doc,SimilarityDefs.p_similarityMethods);
            doc.appendChild(e);
            assertEquals(2,e.getChildNodes().getLength());
            ChemCastObjectFactory.writeXmlFile(doc, "similarity.xml");
            
            NodeList nodes = doc.getElementsByTagName(SimilarityDefs.p_similarityMethods);
            SimilarityDefs sets = new SimilarityDefs((Element)nodes.item(0));
            assertEquals(2,sets.getList().size());
            assertNotNull(sets.get(0).getDescriptors());
            assertEquals("True",sets.get(0).getParameters().get("Test"));
        } catch (Exception x) {
            fail();
        }
    }

    public void testHashtable() {
    	Hashtable<SimilarityModel,String> h = new Hashtable<SimilarityModel,String>();
    	SimilarityModel m = new SimilarityModel();
    	int[] x = new int[1];
    	x[0] =1;
    	m.setSubsetIndices(x);
    	h.put(m,"First");
    	
    	SimilarityModel m1 = new SimilarityModel();
    	m1.setName("new name");
    	int[] x1 = new int[1];
    	x1[0] =1;
    	m1.setSubsetIndices(x1);
    	h.put(m1,"Second");    
    	
    	SimilarityModel m2 = new SimilarityModel();
    	int[] x2 = new int[1];
    	x2[0] =1;
    	m2.setSubsetIndices(x2);
    	assertEquals("First",h.get(m2));
    }    
    public void testHashtable1() {
    	Hashtable<SimilarityDef,String> h = new Hashtable<SimilarityDef,String>();    	

    	SimilarityDef m = new SimilarityDef();    	
    	h.put(m,"First");
    	
    	SimilarityDef m1 = new SimilarityDef();
    	m1.setName("new name");
    	h.put(m1,"Second");    
    	
    	SimilarityDef m2 = new SimilarityDef();
    	assertEquals("First",h.get(m2));
    }
}
