/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.test.weka;

import junit.framework.TestCase;

import org.openscience.cdk.Molecule;

import weka.core.Instance;
import weka.core.Instances;
import weka.gui.InstancesSummaryPanel;
import ambit.data.literature.ReferenceFactory;
import ambit.data.molecule.MoleculesIterator;
import chemCast.application.ChemCASTModule;
import chemCast.similarity.data.ChemCastDescriptor;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;
import chemCast.weka.WekaSimilarityModel;

public class WekaSimilarityModelTest extends TestCase {
	protected WekaSimilarityModel wekaModel ;
	public static void main(String[] args) {
		junit.textui.TestRunner.run(WekaSimilarityModelTest.class);
	}

	public WekaSimilarityModelTest(String arg0) {
		super(arg0);
	}

	protected void setUp() throws Exception {
		super.setUp();
		wekaModel = new WekaSimilarityModel();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		wekaModel = null;
	}

	/*
	 * Test method for 'chemCast.weka.WekaSimilarityModel.process(Object)'
	 */
	public void testProcess() {
		
	}

	/*
	 * Test method for 'chemCast.weka.WekaSimilarityModel.WekaSimilarityModel()'
	 */
	public void testWekaSimilarityModel() {
		WekaSimilarityModel wekaModel = new WekaSimilarityModel();
		assertNull(wekaModel.getDatasets());
		assertNull(wekaModel.getDescriptors());
		assertNotNull(wekaModel.getClassifier());
	}

	/*
	 * Test method for 'chemCast.weka.WekaSimilarityModel.build()'
	 */
	public void testBuild() {
		try {
			final String descr1 = "Descriptor1";
			final String descr2 = "Descriptor2";
			final String descr4 = "Descriptor4";
			final String descr3 = "Class";
			ChemCastDescriptorsList d = new ChemCastDescriptorsList();
			d.add(new ChemCastDescriptor(descr1, ReferenceFactory.createEmptyReference()));
			d.add(new ChemCastDescriptor(descr2,ReferenceFactory.createEmptyReference()));
			d.add(new ChemCastDescriptor(descr4,ReferenceFactory.createEmptyReference()));

			Subsets s = new Subsets();
			Subset s1 = new Subset();
            s1.setName("First");
			s.add(s1);
			MoleculesIterator m = new MoleculesIterator();
			s1.setData(m);
			
			for (int i=0; i < 100; i++ ) {
				Molecule m1 = new Molecule(); //MoleculeFactory.makeAlkane(3+i);
				m1.setProperty(descr1,10*(i+1)+Math.random());
				m1.setProperty(descr2,10*(i+1));
				m1.setProperty(descr3,i % 20 );
				m.addAtomContainer(m1);
			}
            
            Subset s2 = new Subset();
            s2.setName("second");
            s.add(s2);
            MoleculesIterator m2 = new MoleculesIterator();
            s2.setData(m2);
            
            for (int i=0; i < 50; i++ ) {
                Molecule m1 = new Molecule(); //MoleculeFactory.makeAlkane(3+i);
                m1.setProperty(descr1,10*(i+1)+Math.random());
                m1.setProperty(descr2,10+(i+1));
                m1.setProperty(descr3,i % 20 );
                m2.addAtomContainer(m1);
            }           

            
			wekaModel.setDescriptors(d);
			d.get(0).setEnabled(true);
			d.get(1).setEnabled(true);
			d.get(2).setEnabled(true);			
			//wekaModel.setClassProperty(descr3);
			//Classification
			wekaModel.setClassProperty(null);
			wekaModel.setDatasets(s);
			assertNotNull(wekaModel.getDatasets());		
			wekaModel.build();
            
            
			wekaModel.setDatasets(s);
			wekaModel.predict();
			for (int j=0; j < s.size();j++)
				for (int i=0; i < s.get(j).getData().getAtomContainerCount();i++) {
                    /*
					System.out.print(s.get(j).getData().getProperty(i,wekaModel.getClassProperty()));
					System.out.print('\t');
					System.out.print(s.get(j).getData().getProperty(i,"distance"));				
					System.out.print('\t');				
					System.out.println(s.get(j).getData().getProperty(i,wekaModel.getPredictedFieldName()));
                    */
					String[] props = wekaModel.getProperties();
					for (int p=0; p < props.length;p++)
						assertNotNull(s.get(j).getData().getProperty(i,props[p]));

				}
            
            Subsets testSubsets = new Subsets();
            Subset testSubset = new Subset();
            testSubsets.add(testSubset);
            
            MoleculesIterator m3 = new MoleculesIterator();
            testSubset.setData(m3);
            //wekaModel.setClassProperty(descr3);
            for (int i=0; i < 10; i++ ) {
                Molecule m1 = new Molecule(); //MoleculeFactory.makeAlkane(3+i);
                m1.setProperty(descr1,10*(i+1)+Math.random());
                m1.setProperty(descr2,10+(i+1));
                //m1.setProperty(descr3,i % 20 );
                m3.addAtomContainer(m1);
            }  
            for (int i=0; i < 10; i++ ) {
                Molecule m1 = new Molecule(); //MoleculeFactory.makeAlkane(3+i);
                m1.setProperty(descr1,10*(i+1)+Math.random());
                m1.setProperty(descr2,10*(i+1));
                //m1.setProperty(descr3,i % 20 );
                m3.addAtomContainer(m1);
            }            
            wekaModel.setDatasets(testSubsets);
            wekaModel.predict();
            for (int i=0; i < testSubset.getData().getAtomContainerCount();i++) {
                /*
                System.out.print(testSubset.getData().getProperty(i,wekaModel.getClassProperty()));
                System.out.print('\t');
                System.out.print(testSubset.getData().getProperty(i,"distance"));             
                System.out.print('\t');             
                System.out.println(testSubset.getData().getProperty(i,wekaModel.getPredictedFieldName()));
                */
				String[] props = wekaModel.getProperties();
				for (int p=0; p < props.length;p++)
					assertNotNull(testSubset.getData().getProperty(i,props[p]));            	
            	//assertNotNull(testSubset.getData().getProperty(i,wekaModel.getTrainingPrefix()+ "." + wekaModel.getSimilarityProperty()));
                //assertNotNull(testSubset.getData().getProperty(i,wekaModel.getTrainingPrefix()+ "." + wekaModel.getClassificationProperty()));

            }

            /*
            Instances instances = wekaModel.getConvertor().createInstances("test");
            Instance instance = wekaModel.getConvertor().createInstance();
            instances.add(instance);
            instance.setDataset(instances);
            wekaModel.getConvertor().property2Instance(s1, 0, instance);
            wekaModel.predict();
            */
            /*
			
            double value = wekaModel.getClassifier().classifyInstance(instance);
            System.out.println(value);
            */
			
			
		} catch (Exception x) {
			x.printStackTrace();
			fail();
		}
	}

	/*
	 * Test method for 'chemCast.weka.WekaSimilarityModel.predict()'
	 */
	public void testPredict() {

	}

	/*
	 * Test method for 'chemCast.weka.WekaSimilarityModel.getPredictedFieldName()'
	 */
	public void testGetPredictedFieldName() {
			
	}

	/*
	 * Test method for 'chemCast.weka.WekaSimilarityModel.processInstance(Subsets, int, int, Instances, Instance)'
	 */
	public void testProcessInstance() {

	}

	/*
	 * Test method for 'chemCast.weka.WekaSimilarityModel.setDatasets(Subsets)'
	 */
	public void testSetDatasets() {
		wekaModel.setDatasets(new Subsets());
		assertNotNull(wekaModel.getDatasets());		
	}


	/*
	 * Test method for 'chemCast.weka.WekaSimilarityModel.setUseAtomEnvironments(boolean)'
	 */
	public void testSetUseAtomEnvironments() {
		wekaModel.setUseAtomEnvironments(true);
		assertTrue(wekaModel.isUseAtomEnvironments());
		wekaModel.setUseAtomEnvironments(false);
		assertFalse(wekaModel.isUseAtomEnvironments());		
	}

	/*
	 * Test method for 'chemCast.weka.WekaSimilarityModel.setUseFingerprints(boolean)'
	 */
	public void testSetUseFingerprints() {
		wekaModel.setUseFingerprints(true);
		assertTrue(wekaModel.isUseFingerprints());
		wekaModel.setUseFingerprints(false);
		assertFalse(wekaModel.isUseFingerprints());		
	}

	/*
	 * Test method for 'chemCast.weka.WekaSimilarityModel.setDescriptors(ChemCastDescriptorsList)'
	 */
	public void testSetDescriptors() {
		wekaModel.setDescriptors(ChemCASTModule.readDescriptors());
		assertNotNull(wekaModel.getDescriptors());
		wekaModel.setDescriptors(null);
		assertNull(wekaModel.getDescriptors());
	}

	/*
	 * Test method for 'chemCast.weka.WekaSimilarityModel.setClassProperty(String)'
	 */
	public void testSetClassProperty() {
		wekaModel.setClassProperty("property");
		assertEquals("property",wekaModel.getClassProperty());
	}

}


