/* ChemCastListTableModel.java
 * Author: Nina Jeliazkova
 * Date: Sep 23, 2006 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.table.AbstractTableModel;

import chemCast.similarity.data.IChemCastList;
import chemCast.similarity.data.IChemCastObject;

public class ChemCastListTableModel extends AbstractTableModel implements PropertyChangeListener {
    /**
     * 
     */
    private static final long serialVersionUID = 6765738436297033077L;
    protected IChemCastList list;
    protected boolean enabled = true;
    public ChemCastListTableModel(IChemCastList list) {
        super();
        this.list = list;
        if (list != null)
        	list.addPropertyChangeListener(this);
    }

    public int getColumnCount() {
        return 2;
    }

    public int getRowCount() {
        if (list == null) return 0;
        else
        return list.getList().size();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        if ((list == null) || (list.getList()==null)) return "";
        IChemCastObject o = list.getList().get(rowIndex);
        
        switch (columnIndex) {
        case 0: return new Boolean(o.isEnabled());
        case 1: return list.getList().get(rowIndex);
        default : return "";
        }
    }
    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        if (list == null) return ;
        IChemCastObject o = list.getList().get(rowIndex);
        switch (columnIndex) {
        case 0: o.setEnabled(((Boolean) aValue).booleanValue());
        
        default : return ;
        }
    }
    @Override
    public Class<?> getColumnClass(int columnIndex) {
        switch (columnIndex) {
        case 0: return Boolean.class;
        default:
        return super.getColumnClass(columnIndex);
        }
    }
    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return enabled && (columnIndex == 0);
    }
    public void propertyChange(PropertyChangeEvent evt) {
        if ((evt.getPropertyName().equals("add")) ||
                (evt.getPropertyName().equals("remove")) ||
                (evt.getPropertyName().equals("clear")))
            fireTableStructureChanged();
        
    }

    public synchronized IChemCastList getList() {
        return list;
    }

    public synchronized void setList(IChemCastList list) {
        if (this.list != null)
            this.list.removePropertyChangeListener(this);
        this.list = list;
        if (list != null)
        list.addPropertyChangeListener(this);
        fireTableStructureChanged();
    }

	public boolean isEnabled() {
		return enabled;
	}

	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}
}
