/*
Copyright Ideaconsult Ltd. (C) 2005-2007  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package chemCast.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ItemListener;
import java.text.NumberFormat;
import java.util.Hashtable;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import org.openscience.cdk.interfaces.IAtomContainer;

import ambit.ui.data.molecule.Panel2D;
import chemCast.data.PairwiseSimilarityFilter;

public class MatrixToolbar extends JToolBar {

		
		protected StructurePair strucPair;
		protected NNControl nnControl;
		
		/**
		 * 
		 */
		private static final long serialVersionUID = -2741069659513826441L;

		public MatrixToolbar(Action[] show) {
			this(show,true,true);
		}
		public MatrixToolbar(Action[] show, boolean showPair, boolean showControl) {
			super();
			Dimension maxd = new Dimension(Integer.MAX_VALUE,24);
			setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
			strucPair = new StructurePair();
			add(strucPair);
			strucPair.setVisible(showPair);
			nnControl = new NNControl(show);
			add(nnControl);
			nnControl.setVisible(showControl);

		}
		public void setRowStructure(IAtomContainer ac) {
			strucPair.setStructure(0, ac);
		}
		public void setColStructure(IAtomContainer ac) {
			strucPair.setStructure(1, ac);
		}		
		public void setRowNumber(int row) {
			strucPair.setRowNumber(row);
		}
		public void setColNumber(int col) {
			strucPair.setColNumber(col);
		}	
		public void setThreshold(double threshold) {
			nnControl.setThreshold(threshold);
		}
		public double getThreshold()  {
			try {
				return nnControl.getThreshold();
			} catch (Exception x) {
				x.printStackTrace();
				return 0.8;
			}
		}
		public String getCondition()  {
			return nnControl.getCondition();
		}		
		public boolean queryFromRow() {
			return strucPair.queryFromRow();	
		}
		public void setSimilarity(Object similarity) {
			strucPair.setSimilarity(similarity);
		}
		public boolean getNNFromRows() {
			return nnControl.isSearchRows();
		}
		public boolean getNNFromCols() {
			return !nnControl.isSearchRows();
		}	
		public void setRowDatasetName(String name) {
			strucPair.setDatasetName(0,name);
			nnControl.setRowName(name);
			
		}
		public void setColumnDatasetName(String name) {
			strucPair.setDatasetName(1,name);
			nnControl.setColName(name);
		}	
	}

class NNControl extends JPanel {
	protected JRadioButton[] dataset_selection;
	protected JFormattedTextField thresholdField;
	protected JComboBox thresholdConditions;
	protected JSlider thresholdSlider;

	public NNControl(Action[] show) {
		super();
		Dimension prefDimension = new Dimension(64,24);
		Dimension minDimension = new Dimension(32,24);
		
		setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
		
		int row = 0;

		
	
		dataset_selection = new JRadioButton[2];
		ButtonGroup bg = new ButtonGroup();
		for (int i=0; i < dataset_selection.length; i++) {
			JPanel selectionPanel = new JPanel();
			selectionPanel.setLayout(new BorderLayout());	

			dataset_selection[i] = new JRadioButton(); 
			//new JLabel(">>",UITools.createImageIcon("/ambit/ui/images/nav_prev.gif"),JLabel.LEFT);
			bg.add(dataset_selection[i]);
			dataset_selection[i].setSelected(i==0);
			dataset_selection[i].setPreferredSize(new Dimension(200,32));
			dataset_selection[i].setMinimumSize(minDimension);
			dataset_selection[i].setAlignmentX(LEFT_ALIGNMENT);
			selectionPanel.add(dataset_selection[i],BorderLayout.CENTER); //inefficient, but works
			add(selectionPanel);
	        row++;    
		}

		JPanel simPanel = new JPanel();
		simPanel.setLayout(new BoxLayout(simPanel,BoxLayout.LINE_AXIS));
		JLabel tlabel = new JLabel("With similarity ");
		tlabel.setPreferredSize(new Dimension(100,24));
		tlabel.setMinimumSize(minDimension);
		simPanel.add(tlabel);		

		thresholdConditions = new JComboBox(PairwiseSimilarityFilter.conditions);
		thresholdConditions.setSelectedIndex(1);
		thresholdConditions.setPreferredSize(prefDimension);
		thresholdConditions.setMinimumSize(minDimension);
		thresholdConditions.setEditable(false);
		thresholdConditions.addItemListener(new ItemListener() {
			public void itemStateChanged(java.awt.event.ItemEvent e) {
				
				Object[] s = e.getItemSelectable().getSelectedObjects();
			};
		});
		simPanel.add(thresholdConditions);
        
        
		thresholdField = new JFormattedTextField(NumberFormat.getNumberInstance());
		thresholdField.setValue(new Double(0.8));
		thresholdField.setPreferredSize(prefDimension);
		thresholdField.setMinimumSize(minDimension);
		thresholdField.setEditable(false);
		simPanel.add(thresholdField);
        
		add(simPanel);
		
		thresholdSlider = new JSlider(JSlider.HORIZONTAL,0,100,(int)(0.8*100));
		thresholdSlider.setMajorTickSpacing(10);
		thresholdSlider.setPaintTicks(true);

		thresholdSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
			    JSlider source = (JSlider)e.getSource();
			    if (!source.getValueIsAdjusting()) {
			        int threshold = source.getValue();
			        setThreshold(((double)threshold) / 100.0);
			    }
			}
		});
		//Create the label table
		Hashtable labelTable = new Hashtable();
		labelTable.put( new Integer( 0 ), new JLabel("Different") );
		labelTable.put( new Integer( 50 ), new JLabel("Similar") );
		labelTable.put( new Integer( 100 ), new JLabel("Same") );
		thresholdSlider.setLabelTable( labelTable );

		thresholdSlider.setPaintLabels(true);
		
		thresholdSlider.setToolTipText("Similarity threshold");
		add(thresholdSlider);
        row++;

		JPanel clrPanel = new JPanel();
		clrPanel.setLayout(new BoxLayout(clrPanel,BoxLayout.LINE_AXIS));
		
        JLabel legend = new JLabel("Color coding:");
        legend.setPreferredSize(new Dimension(64,24));
        clrPanel.add(legend);
        
		AbstractTableModel model = new AbstractTableModel() {
			public int getColumnCount() {
				return 12;
			}
			public int getRowCount() {
				return 1;
			}
			public Object getValueAt(int rowIndex, int columnIndex) {
				switch (columnIndex) {
				case 0: return "0";
				case 11:return "1";
				default: return new Double((double)(columnIndex-1)/(getColumnCount()-3));
				}
				
			}
		};
		JTable table = new JTable(model) {
        	public void createDefaultColumnsFromModel() {
        		
        		TableModel model = getModel();
                // Remove any current columns
                TableColumnModel cm = getColumnModel();
                while (cm.getColumnCount() > 0) {
                          cm.removeColumn(cm.getColumn(0));
                }
        		
                addColumn(new TableColumn(0));
        		TableCellRenderer r = new MatrixCellRenderer();
                for (int i=1; i < model.getColumnCount()-1;i++) {
                    TableColumn c = new TableColumn(i);
                    c.setMinWidth(10);
                    c.setMaxWidth(10);
                    c.setHeaderValue(new Integer(i-1));
                    c.setPreferredWidth(10);
                    c.setCellRenderer(r);
                    addColumn(c);
                }
                addColumn(new TableColumn(model.getColumnCount()-1));
        	}
        };
        table.setPreferredSize(new Dimension(150,24));
        clrPanel.add(table);                
        
        add(clrPanel);
        row++;
        
		if (show != null) {
			clrPanel = new JPanel();
			clrPanel.setLayout(new BoxLayout(clrPanel,BoxLayout.LINE_AXIS));     
				
	        for (Action action: show) {
		        JButton button = new JButton(action);
		        button.setPreferredSize(new Dimension(100,24));
		        clrPanel.add(button);
	        }
	        add(clrPanel);
		}
       

		setBorder(BorderFactory.createTitledBorder("Find most similar chemicals from dataset:"));
	}
	public boolean isSearchRows() {
		return dataset_selection[0].isSelected();
	}
	public void setColName(String name) {
		setDatasetName(1, name);
	}	
	
	public void setRowName(String name) {
		setDatasetName(0, name);
	}	

	protected void setDatasetName(int index,String name) {
		dataset_selection[index].setText(name);
		dataset_selection[index].setToolTipText(name);
	}	
	public double getThreshold() {
		return ((Number)thresholdField.getValue()).doubleValue();
	}
	public void setThreshold(double threshold) {
		thresholdField.setValue(threshold);
	}
	public String getCondition() {
		return thresholdConditions.getSelectedItem().toString();
	}

}

class StructurePair extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -1256957683602294518L;
	protected Panel2D[] panel2d;
	protected JLabel[] dataset_title;
	protected JFormattedTextField[] dataset_index;
	protected JRadioButton[] structure_selection;
	
	protected JFormattedTextField similarityValue;
	protected JLabel similarityLabel;
	

	public StructurePair() {
		super();

		Dimension minDimension = new Dimension(64,24);
		panel2d = new Panel2D[2];
		dataset_title = new JLabel[2];
		dataset_index = new JFormattedTextField[2];
		structure_selection = new JRadioButton[2];
		
		setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
		int row = 0;
		
		ButtonGroup bg = new ButtonGroup();
		for (int i=0; i < panel2d.length; i++) {
			panel2d[i] = new Panel2D();
			
			panel2d[i].setPreferredSize(new Dimension(100,100));

			panel2d[i].setBorder(BorderFactory.createTitledBorder(Integer.toString(i)));

			add(panel2d[i]);

			JPanel selectionPanel = new JPanel();
			selectionPanel.setLayout(new BoxLayout(selectionPanel,BoxLayout.LINE_AXIS));
			
			
			structure_selection[i] = new JRadioButton(); 
				//new JLabel(">>",UITools.createImageIcon("/ambit/ui/images/nav_prev.gif"),JLabel.LEFT);
			bg.add(structure_selection[i]);
			structure_selection[i].setSelected(i==0);
            structure_selection[i].setPreferredSize(new Dimension(32,32));
            structure_selection[i].setMinimumSize(new Dimension(24,24));
            
            selectionPanel.add(structure_selection[i]);
            
			dataset_title[i] = new JLabel();
			dataset_title[i].setEnabled(false);
			dataset_title[i].setPreferredSize(new Dimension(150,24));
			dataset_title[i].setMinimumSize(new Dimension(64,24));
			
			selectionPanel.add(dataset_title[i]);

            
            dataset_index[i] = new JFormattedTextField(NumberFormat.getIntegerInstance());
            dataset_index[i].setEnabled(false);
			dataset_index[i].setPreferredSize(minDimension);
			dataset_index[i].setMinimumSize(new Dimension(24,24));
			
			selectionPanel.add(dataset_index[i]);
            
            row = row + 2;
            add(selectionPanel);
		}
		
		JPanel simPanel = new JPanel();
		simPanel.setLayout(new BoxLayout(simPanel,BoxLayout.LINE_AXIS));
		
		similarityLabel = new JLabel("Similarity");
		similarityLabel.setPreferredSize(new Dimension(150,24));

		simPanel.add(similarityLabel);

		
		similarityValue = new JFormattedTextField(NumberFormat.getNumberInstance());
		similarityValue.setEditable(false);
		similarityValue.setOpaque(true);
		similarityValue.setPreferredSize(minDimension);
		
        simPanel.add(similarityValue);
        add(simPanel);
        
        row++;
        
		
		
        setBorder(BorderFactory.createTitledBorder("Selected pair"));
	}
	public void setColNumber(int number) {
		setStrucNumber(1, number);
	}	
	public void setRowNumber(int number) {
		setStrucNumber(0, number);
	}
	protected void setStrucNumber(int index, int number) {
		if (number <= 0)
			dataset_index[index].setText("");
		else 
			dataset_index[index].setValue(number);
	}	
	public void setColName(String name) {
		setDatasetName(1, name);
	}	
	
	public void setRowName(String name) {
		setDatasetName(0, name);
	}	

	protected void setDatasetName(int index,String name) {
		dataset_title[index].setText(name);
		dataset_title[index].setToolTipText(name);
	}	
	public boolean queryFromRow() {
		return structure_selection[0].isSelected();
	}
	public void setSimilarity(Object number) {
		similarityValue.setValue(number);
	}
	public void setStructure(int index, IAtomContainer ac) {
		panel2d[index].setAtomContainer(ac, true);
	}
}
