/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.ui;

import java.awt.Color;
import java.awt.Paint;
import java.awt.Shape;

import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;

import ambit.processors.results.AbstractSimilarityMatrix;
import chemCast.similarity.data.Subsets;

class MoleculesXYZRenderer extends XYLineAndShapeRenderer {
    protected Shape fileShape;
    protected Shape subsetShape;
    protected Subsets subsets;
    protected Object colorProperty=null;
    /**
     * 
     */
    private static final long serialVersionUID = -777652084545420243L;
    public MoleculesXYZRenderer(Subsets subsets,Shape fileShape, Shape subsetShape) {
        super(false,true);
        this.fileShape = fileShape;
        this.subsetShape = subsetShape;
        this.subsets = subsets;
    }
    @Override
    public Paint getSeriesFillPaint(int series) {
        return getSeriesPaint(series);
        
    }
    @Override
    public Paint getItemPaint(int series, int item) {
        if ((colorProperty==null) || ("groups".equals(colorProperty))) return getSeriesFillPaint(series);
        else {
            try {
                Object value = subsets.get(series).getData().getProperty(item, colorProperty);
                if (value instanceof Number)
                    return AbstractSimilarityMatrix.valueToColor((Number)value);
                else 
                    return AbstractSimilarityMatrix.valueToColor(new Float(value.toString()));
            } catch (Exception x) {
                return Color.gray;
            }
        }
    }
    @Override
    public Paint getSeriesPaint(int series) {
    	try {
            return subsets.get(series).getColor();
    	} catch (Exception x) {
    		return Color.BLACK;
    	}
    }
    @Override
    public Shape getItemShape(int series,int item) {
        return getSeriesShape(series);
    }
    @Override
    public Shape getSeriesShape(int series) {
        if (series ==0) return fileShape; else return subsetShape;
    }
    @Override
    public Boolean getSeriesVisible(int series) {
    	try {
    		return subsets.get(series).isEnabled();
    	} catch (Exception x) {
    		return false;
    	}
    }
    @Override
    public boolean getItemVisible(int series, int item) {
        return getSeriesVisible(series);
        
    }
    public synchronized Object getColorProperty() {
        return colorProperty;
    }
    public synchronized void setColorProperty(Object colorProperty) {
        this.colorProperty = colorProperty;
        
    }
    
}


