/**
 * 
 */
package chemCast.ui.actions;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.Icon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.io.IChemObjectWriter;

import ambit.data.molecule.IAtomContainersList;
import ambit.data.molecule.PropertyTranslator;
import ambit.io.FileInputState;
import ambit.io.FileOutputState;
import ambit.io.FileWithHeaderWriter;
import ambit.io.MDLWriter;
import ambit.io.MyIOUtilities;
import ambit.misc.AmbitCONSTANTS;
import ambit.ui.UITools;
import ambit.ui.data.SelectPropertiesPanel;
import chemCast.application.ChemCASTModule;
import chemCast.application.StatusBar;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;

/**
 * @author Nina Jeliazkova nina@acad.bg
 *
 */
public class FileSaveAction extends ToxmatchAction {

	/**
	 * 
	 */
	private static final long serialVersionUID = -3913058195190818916L;
	/**
	 * @param userData
	 * @param mainFrame
	 * @param dataset
	 */
	public FileSaveAction(ChemCASTModule userData, JFrame mainFrame, int dataset) {
		this(userData, mainFrame, "Save",dataset);
	}

	/**
	 * @param userData
	 * @param mainFrame
	 * @param name
	 * @param dataset
	 */
	public FileSaveAction(ChemCASTModule userData, JFrame mainFrame,
			String name, int dataset) {
		super(userData, mainFrame, name, UITools.createImageIcon("ambit/ui/images/save_16.png"), dataset);
	}

	/**
	 * @param userData
	 * @param mainFrame
	 * @param name
	 * @param icon
	 * @param dataset
	 */
	public FileSaveAction(ChemCASTModule userData, JFrame mainFrame,
			String name, Icon icon, int dataset) {
		super(userData, mainFrame, name, icon, dataset);
	}

	/* (non-Javadoc)
	 * @see chemCast.ui.actions.ChemCASTAction#processDataset(chemCast.similarity.data.Subsets)
	 */
	@Override
	protected void processDataset(Subsets data) throws Exception {
	    if (data.size()==0) {
            getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"No dataset available!");
            return;
        }
        ArrayList<String> fields;
        DatasetDef parentDataset = data.getParentDataset();
        File file = MyIOUtilities.selectFile(mainFrame,null,
                getChemCastData().getDefaultDir(),
                FileInputState.extensions,FileInputState.extensionDescription,false);
        
        
        if (file == null) return;
        
        getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Saving to file "+file.getAbsolutePath());
        getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Processing,null,new Boolean(true));
        
        try {
        	
	        IChemObjectWriter writer = FileOutputState.getWriter(new FileOutputStream(file), file.getName());
            
            
            PropertyTranslator properties = parentDataset.getProperties();
            SelectPropertiesPanel sp = new SelectPropertiesPanel(properties,new String[] {"identifiers","descriptors","endpoint","similarity","subsets"});
            if (JOptionPane.showConfirmDialog(mainFrame,sp,"Select data fields to save",JOptionPane.OK_CANCEL_OPTION,JOptionPane.PLAIN_MESSAGE)       
                    ==JOptionPane.OK_OPTION) {
                fields = sp.getFields();
                data.setFields(fields);
            } else fields = new ArrayList<String>(); 
            
            /*
	        boolean parentDatasetOnly = false;
	        for (int i=0; i < data.size(); i++)
	        	if ((data.get(i).isEnabled())&& (data.get(i) instanceof DatasetDef)) {
	        		processSubset(data.get(i),writer);
	        		parentDatasetOnly = true;
	        		break;
	        	}
	        if (!parentDatasetOnly) {		
		        for (int i=0; i < data.size(); i++)
		        	if (data.get(i).isEnabled())
		        		processSubset(data.get(i),writer);
	        }
            */
            parentDataset.write(writer, fields);
	        writer.close();
        } catch (Exception x) {
    		getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Error,null,x.getMessage());        	
        }
        getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Processing,null,new Boolean(false));        
        getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"");        

	}
    /*
	protected void processSubset(Subset subset, IChemObjectWriter writer, ArrayList fields) {
		try {
            Hashtable p = new Hashtable(); 
	        IAtomContainersList data = subset.getData();
	        getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Status,null,"Saving " + data.toString() + " to " + writer.toString());
	        for (int i=0; i < data.getAtomContainerCount(); i++)
	             try {
	            	 getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Progress,null,new Integer(100*i/data.getAtomContainerCount()));
	                 IAtomContainer a = data.getAtomContainer(i);
                     p.clear();
                     for (int f=0; f < fields.size();f++) {
                         Object value = data.getProperty(i,fields.get(f));
                         if (value != null)
                             p.put(fields.get(f), value);
                     }    

                     if (writer instanceof MDLWriter) {
                         ((MDLWriter) writer).setSdFields(p);
                     } else if (writer instanceof FileWithHeaderWriter) {
                         ((FileWithHeaderWriter)writer).setHeader(fields);
                     }
	                 writer.write(a);
	             } catch (Exception x) {
	            	 getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Error,null,x);
	             }
		} catch (Exception x) {
			getChemCastData().getPropertyChangeSupport().firePropertyChange(StatusBar.Message_Error,null,x);			
		}
	}
    */

}
