/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.ui.wizard;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import ambit.data.IAmbitEditor;
import ambit.data.molecule.PropertyTranslator;
import ambit.ui.EditorPanel;
import chemCast.data.IObjectFilter;
import chemCast.data.PropertyFilter;
import chemCast.data.RangePropertyFilter;
import chemCast.exception.SubsetException;
import chemCast.similarity.data.DatasetDef;
import chemCast.similarity.data.Subset;
import chemCast.similarity.data.Subsets;
import chemCast.ui.IKeyValuesUpdater;
import chemCast.ui.ObjectPropertyFilterEditor;
import chemCast.ui.RadioBoxPanel;

import com.nexes.wizard.WizardPanelDescriptor;

public class SubsetWizard extends WizardWrapper implements IKeyValuesUpdater {
    protected DatasetDef dataset;
	protected Subsets subsets;
    protected Subset subset;
	//protected IObjectFilter filter = null;
	protected List<IObjectFilter> filters = null;
	protected EditorPanel editorPanel;
    protected EditorPanel subsetEditorPanel;
	ChemCastWizardPanelDescriptor descriptor1;
    ChemCastWizardPanelDescriptor descriptor2;
    ChemCastWizardPanelDescriptor descriptor3;
    protected static final String[] titles = {
        "Create an empty group",
        "Create a group by parameter value",
        "Create a group by parameter range",
        "Create groups for all available parameter values"        
    };
	public SubsetWizard(Container frame, Subsets subsets, DatasetDef dataset) {
		super(frame);
        this.dataset = dataset;
		this.subsets = subsets;
        try {
            subset = new Subset();
            
            subsets.add(subset);
            filters = new ArrayList<IObjectFilter>();
        } catch (Exception x) {
            
        }
		createPanels();
	}

	public void createPanels() {
		List<String> list = new ArrayList<String>();
        for (int i=0; i < titles.length;i++)
            list.add(titles[i]);

		RadioBoxPanel options = new RadioBoxPanel("Define new subset",list,0){
		     public void selectObject(ActionEvent e, Object object) {
                    if (titles[0].equals(object))
                    	descriptor1.setNextId("subsetedit");
	                else {
	                	descriptor1.setNextId("filteredit");
	                	try {
	                		filters.clear();
		                	if (titles[1].equals(object)) filters.add(new PropertyFilter());
                            else if (titles[2].equals(object)) filters.add(new RangePropertyFilter());
                            else if (titles[3].equals(object)) {
                            	filters.add(new PropertyFilter());
                            	filters.add(new PropertyFilter());
                            }
		                	IAmbitEditor editor = filters.get(0).getEditor();
                            
		                	editor.setEditable(true);
                            if (editor instanceof ObjectPropertyFilterEditor) { 
                                ((ObjectPropertyFilterEditor) editor).addTags(
                                        dataset.getProperties().enumerateProperties(new String[]{
                                                PropertyTranslator.type_descriptors,
                                                PropertyTranslator.type_results,
                                                "similarity",
                                                "subsets",
                                                PropertyTranslator.type_identifiers
                                                },
                                                ""));
                                setKeyValuesUpdater((ObjectPropertyFilterEditor) editor);
                            }
                            
		                	editorPanel.setEditor(editor);
	                	} catch (Exception x) {
	                		x.printStackTrace();
	                	}
	                }	
	            }
		};
		
        descriptor1 = new ChemCastWizardPanelDescriptor("subsettypes","subsetedit",null,options,null);
        registerWizardPanel(descriptor1.getId().toString(), descriptor1);
        
        editorPanel = new EditorPanel();
        descriptor2 = new ChemCastWizardPanelDescriptor("filteredit","subsetedit","subsettypes",editorPanel,null) {
        	public void aboutToHidePanel() {
        		switch (filters.size()) {
        		case 0:return;
        		case 1: {
	        		IObjectFilter filter = filters.get(0);
	                if (filter !=null) {
	                    try {
	                        
	                        subset.setFilterClass(filter.getClass().getName());
	                        
	                		subset.setProperty(Subset.p_tag, filter.getProperty(Subset.p_tag));
	                		subset.setProperty(Subset.p_value, filter.getProperty(Subset.p_value));
	                		subset.setProperty(RangePropertyFilter.p_maxValue, filter.getProperty(RangePropertyFilter.p_maxValue));
	                		subset.setProperty(RangePropertyFilter.p_minValue, filter.getProperty(RangePropertyFilter.p_minValue));
	                        subset.setName(filter.toString());
	                        subsetEditorPanel.setEditor(subset.getEditor());
	                    } catch (Exception x) {
	                        x.printStackTrace();
	                        subset = null;
	                    }
	                }
	                break;
        			}
        		default: {
        			subsets.clear();
        			subset = null;
        			Collection values = getValuesByField(filters.get(0).getProperty(Subset.p_tag));
        			IObjectFilter filter = filters.get(0);
        			Iterator vi = values.iterator();
        			while (vi.hasNext()) {
        				Object value = vi.next();
        				try {
        				Subset s = new Subset();
	                        s.setFilterClass(filter.getClass().getName());
	                        
	                		s.setProperty(Subset.p_tag, filter.getProperty(Subset.p_tag));
	                		s.setProperty(Subset.p_value, value);
	                		s.setProperty(RangePropertyFilter.p_maxValue, filter.getProperty(RangePropertyFilter.p_maxValue));
	                		s.setProperty(RangePropertyFilter.p_minValue, filter.getProperty(RangePropertyFilter.p_minValue));
	                        s.setName(filter.getProperty(Subset.p_tag) + "=" +value );
	                        subsets.add(s);
	                        
        				} catch (SubsetException x) {
        					x.printStackTrace();
        				}
                          				
        			}
        			subsetEditorPanel.setEditor(subsets.getEditor());      
        		}
        		}
        	};
        };
        registerWizardPanel(descriptor2.getId().toString(), descriptor2);
        //IAmbitEditor editor = subset.getEditor();

        subsetEditorPanel = new EditorPanel();
        descriptor3 = new ChemCastWizardPanelDescriptor("subsetedit",WizardPanelDescriptor.FINISH,"subsettypes",
                subsetEditorPanel,null);
        
        registerWizardPanel(descriptor3.getId().toString(), descriptor3);
        
        setCurrentPanel(descriptor1.getId().toString());
	}
    /**
     * Fills in values combobox based on key value.
     */
    public Collection getValuesByField(Object field) {
        try {
            return dataset.getData().getValuesPerField(field, 100);
        } catch (Exception x) {
            x.printStackTrace();
            return null;
        }
    }
    protected void setKeyValuesUpdater(ObjectPropertyFilterEditor editor) {
        editor.setKeyValuesUpdater(this);
    }
}


