/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.weka;

import java.util.List;

import org.openscience.cdk.qsar.model.QSARModelException;

import weka.clusterers.Clusterer;
import weka.clusterers.SimpleKMeans;
import weka.core.Instance;
import weka.core.Instances;
import weka.filters.Filter;
import weka.filters.MultiFilter;
import weka.filters.unsupervised.attribute.ReplaceMissingValues;
import ambit.data.IAmbitEditor;
import ambit.data.molecule.IAtomContainersList;
import ambit.exceptions.AmbitException;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.Subsets;

public class WekaClustering extends AbstractWekaModel  {
	protected Instances pairwiseSimilarity = null;
	/**
	 * 
	 */
	private static final long serialVersionUID = -6676971982950181782L;
	protected Clusterer clusterer = null;

	public WekaClustering() {
		super();
		convertor.setUseClassProperty(false);
		clusterer = new SimpleKMeans();
		//Clusterer clusterer = new MakeDensityBasedClusterer();
		try {
			((SimpleKMeans)clusterer).setOptions(new String[]{"-N","5"});
		} catch (Exception x) {
			x.printStackTrace();
		}
	}


	public float predict(IAtomContainersList object1, int index1,
			IAtomContainersList object2, int index2) throws AmbitException {
		if (clusterer == null) throw new AmbitException("Classifier not assigned!");
    	try {
    		if (pairwiseSimilarity==null) pairwiseSimilarity = convertor.createInstances("pairwise similarity");
	    	Instance instance1 = convertor.createInstance();
	    	instance1.setDataset(pairwiseSimilarity);
	    	convertor.property2Instance(object1,index1,instance1,"?");
	    	Instance instance2 = convertor.createInstance();
	    	instance2.setDataset(pairwiseSimilarity);
	    	convertor.property2Instance(object2,index2,instance2,"?");	 
			double clsLabel1 = clusterer.clusterInstance(instance1);
			double clsLabel2 = clusterer.clusterInstance(instance2);
			if (clsLabel1 == clsLabel2) return 1;
			else return 0;
    	} catch (Exception x) {
    		throw new AmbitException(x);
    	}
	}

	@Override
	protected void build(Instances instances) throws Exception {
		/*
		instances.setClassIndex(instances.numAttributes() - 1);
		weka.filters.unsupervised.attribute.Remove filter = new weka.filters.unsupervised.attribute.Remove();
		filter.setAttributeIndices("" + (instances.classIndex() + 1));
		filter.setInputFormat(instances);
		Instances dataClusterer = Filter.useFilter(instances, filter);
		*/
        
        MultiFilter multiFilter = new MultiFilter();
        multiFilter.setFilters(new Filter[] {
                new ReplaceMissingValues()
                });
        multiFilter.setInputFormat(instances);
        Instances newInstances = Filter.useFilter(instances, multiFilter);
		clusterer.buildClusterer(newInstances);
		
		/*
		ClusterEvaluation eval = new ClusterEvaluation();
		eval.evaluateClusterer(instances);
		System.out.println(eval.clusterResultsToString());
		*/

		setPredicting(true);
	}
	public void build() throws QSARModelException {
		if (clusterer == null) throw new QSARModelException("Classifier not assigned!");
		super.build();
	}

	public void predict() throws QSARModelException {
		if (clusterer == null) throw new QSARModelException("Classifier not assigned!");
		try {
             
			super.predict();
		} catch (Exception x) {
			throw new QSARModelException(x.getMessage());
		}
		setDatasets(null);

	}
	public void processInstance(Subsets datasets, int subsetIndex, int objectIndex, Instances instances, Instance instance) throws Exception {
  		  if (!isPredicting()) return;
		  instance.setDataset(instances);
		  double clsLabel = clusterer.clusterInstance(instance);
		  double[] d = clusterer.distributionForInstance(instance);
		  IAtomContainersList data = datasets.get(subsetIndex).getData();
		  if (data == null) return;
		  String property = getTrainingPrefix() + "." +  getClassificationProperty();
		  data.setProperty(objectIndex, property,clsLabel);
		  property += ".";
		  for (int i=0; i<d.length;i++) 
			  data.setProperty(objectIndex, property + (i+1),d[i]);
          
          if (modelStatistics ==null)  modelStatistics = createModelStatistics(true,datasets.toString());
          modelStatistics.addValue(datasets.toString(),datasets.get(subsetIndex).toString(), 
                  Long.toString(Math.round(clsLabel)));
	}

	public ChemCastDescriptorsList getDescriptors() {
		return convertor.getDescriptors();
	}
	public void setDescriptors(ChemCastDescriptorsList descriptors) {
		if (descriptors != null)
	        for (int i=0; i < descriptors.getList().size();i++)
	            descriptors.get(i).setEnabled(true);
		convertor.setDescriptors(descriptors);
//        setPredicting(false);
	}
	@Override
	public void setParameter(Object parameter, Object value) {
		super.setParameter(parameter, value);
		try {
			if (parameter.equals("cluster")) {
				int n = Integer.parseInt(value.toString());
				((SimpleKMeans)clusterer).setNumClusters(n);
			}
		} catch (Exception x) {
			x.printStackTrace();
		}
	}
	@Override
	public String toString() {
		return getStatus(false);
	}
	@Override
	public String getStatus(boolean verbose) {
		try {
            
			StringBuffer b = new StringBuffer();
            if (verbose) b.append("Nearest neighbors ");
            b.append("Clustering");
	        b.append("\nDescriptors\n");
	        
	        ChemCastDescriptorsList d = getDescriptors();
	        List slist = d.getList();
	        for (int i=0; i < slist.size(); i++) {
	        	b.append(d.get(i));
	        	b.append("\n");
	        }	
	        b.append("\n");

			b.append(clusterer.toString());
			return b.toString();
		} catch (Exception x) { return "Clustering"; }
	}


    public synchronized Clusterer getClusterer() {
        return clusterer;
    }


    public synchronized void setClusterer(Clusterer clusterer) {
        this.clusterer = clusterer;
    }
    @Override
    public IAmbitEditor getEditor() {
        return new WekaClusteringEditor(this);
    }
    public String[] getProperties() {
    	return new String[]{
    	getTrainingPrefix() + "." + getClassificationProperty()};
    }

}

class WekaClusteringEditor extends WekaSimilarityModelEditor {
    /**
     * 
     */
    private static final long serialVersionUID = -173937342874902993L;
    
    public WekaClusteringEditor(WekaClustering model) {
        super(model);
        
    }

    @Override
    protected Object getObject() {
        return ((WekaClustering)model).getClusterer();
    }
}

