/*
Copyright (C) 2005-2006  

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*/

package chemCast.weka;

import java.awt.Component;
import java.util.List;

import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.qsar.model.QSARModelException;

import weka.classifiers.Classifier;
import weka.classifiers.Evaluation;
import weka.classifiers.lazy.IBk;
import weka.classifiers.meta.FilteredClassifier;
import weka.core.DistanceFunction;
import weka.core.EuclideanDistance;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.NearestNeighbourSearch;
import weka.filters.Filter;
import weka.filters.MultiFilter;
import weka.filters.supervised.attribute.AttributeSelection;
import weka.filters.unsupervised.attribute.RemoveUseless;
import weka.filters.unsupervised.attribute.ReplaceMissingValues;
import weka.filters.unsupervised.attribute.Standardize;
import ambit.data.IAmbitEditor;
import ambit.data.molecule.IAtomContainersList;
import ambit.exceptions.AmbitException;
import ambit.similarity.IDistanceFunction;
import chemCast.similarity.data.ChemCastDescriptorsList;
import chemCast.similarity.data.Subsets;

 
public class WekaSimilarityModel extends AbstractWekaModel{
	/**
	 * 
	 */
	private static final long serialVersionUID = -2380193857083098578L;
	
	protected FilteredClassifier filteredClassifier;
	protected boolean descriptorSelection = false;
	
	protected Evaluation evaluation_crossv;
	protected Evaluation evaluation_model;
	
    protected Filter standardization;
    protected AttributeSelection pca;
    protected AttributeSelection bestAttributes;
    protected Instances pairwiseSimilarity = null;
    protected boolean doesPrediction = true;
    protected boolean xval = false;
    protected String[] classValues;
	
	public WekaSimilarityModel() {
		super();
        setPredicting(false);
		filteredClassifier = new FilteredClassifier();
		//-F "weka.filters.supervised.attribute.AttributeSelection -E \"weka.attributeSelection.ReliefFAttributeEval -M -1 -D 1 -K 10 -A 2\" -S \"weka.attributeSelection.Ranker -T -1.7976931348623157E308 -N -1\"" 
		//-W weka.classifiers.lazy.IBk -- -K 1 -W 0 -X -E -A "weka.core.KDTree -A weka.core.EuclideanDistance -W 0.01 -L 40"
		Classifier nn = new IBk(3);
		String[] options = new String[] {
			"-K","3",
			"-W","0",
            "-I", //weight by 1-distance
			//"-I", //weight by 1/distance
			//"-X", //crossvalidation
			//"-E", //mean square 
			"-A", //algorithm
			"weka.core.KDTree -A weka.core.EuclideanDistance -W 0.01 -L 40"
		};
		try {
			nn.setOptions(options);
		} catch (Exception x) {
			x.printStackTrace();
		}
		filteredClassifier.setClassifier(nn);
		
		MultiFilter multiFilter = new MultiFilter();
        /*
         * Using this filter gives Filter didn't make the test instance immediately available!
         */
		/*
		Remove removeIDs = new Remove();
		removeIDs.setAttributeIndices("1");
		*/
        //      remove.setInputFormat(inst);
        
		standardization = new Standardize();

		pca = new AttributeSelection();
		options = new String[] {
				"-E",
				//"weka.attributeSelection.PrincipalComponents -R 0.95 -A -1 -O", transform back to original space
                "weka.attributeSelection.PrincipalComponents -R 0.9",
				"-S",
				"weka.attributeSelection.Ranker -T -1.7976931348623157E308 -N -1"
			};
		try {
			pca.setOptions(options);
		} catch (Exception x) {
			x.printStackTrace();
		}
		
		bestAttributes = new AttributeSelection();
		options = new String[] {
				"-E",
				"weka.attributeSelection.ReliefFAttributeEval -M -1 -D 1 -K 10 -A 2",
				"-S",
				"weka.attributeSelection.Ranker -T -1 -N -1"
			};
		try {
            bestAttributes.setOptions(options);
		} catch (Exception x) {
			x.printStackTrace();
		}
			
		//multiFilter.setFilters(new Filter[] {standardization,bestAttributes});
        multiFilter.setFilters(new Filter[] {//removeIDs,
        		new RemoveUseless(),
        		new ReplaceMissingValues(),
        		standardization,pca,bestAttributes});
		filteredClassifier.setFilter(multiFilter);
		
	}
	
	/**
	 *  Filter the dataset using weka.filters.supervised.instance.SpreadSubsample,
  -M 1. This should give you a dataset with uniform class
  distributions.

	 */
	@Override
	protected void build(Instances instances) throws Exception {
        //instances.classAttribute().name()+".predicted";
		//properties2Instances(descriptors, null, useFingerprints, datasets);
		classValues = null;
		doesPrediction = ! instances.classAttribute().isNominal();
		Classifier classifier = filteredClassifier;
		
		if (filteredClassifier.getFilter() ==null)	classifier = getClassifier();
        
        if (instances.numInstances() <= ((IBk)getClassifier()).getKNN() ) {
            ((IBk)getClassifier()).setKNN(instances.numInstances());
            //((IBk)getClassifier()).setCrossValidate(true);
        }
		classifier.buildClassifier(instances);
		
		if (instances.classAttribute().isNominal() && (instances.classAttribute().numValues()>0)) {
			classValues = new String[instances.classAttribute().numValues()];
			for (int i=0; i < classValues.length ; i++ ) {
				classValues[i] = instances.classAttribute().value(i);
			}	
		}
		
        evaluation_crossv = new Evaluation(instances);
        evaluation_model = new Evaluation(instances);
        
        
        //int n = instances.numInstances();
        /*
        System.out.println(filteredClassifier.getFilter().toString());
        System.out.println(classifier.toString());
        */
        try {
            evaluation_model.evaluateModel(classifier, instances);
            //System.out.println(evaluation_model.toSummaryString());
        } catch (Exception x) {
            x.printStackTrace();
        }

        try {
        	if (xval) {
	        	evaluation_crossv.crossValidateModel(classifier,instances,10,instances.getRandomNumberGenerator(1));
	            //System.out.println(evaluation_crossv.toSummaryString());
        	}
        } catch (Exception x) {
            x.printStackTrace();
        }
        
      
		
	}
	public void predict() throws QSARModelException {
		if (filteredClassifier == null) throw new QSARModelException("Classifier not assigned!");
		if (evaluation_model == null) throw new QSARModelException("Model not built!");
		try {
			super.predict();

//			convertor.setInstanceProcessor(this);
//			convertor.properties2Instances(datasets);
		} catch (Exception x) {
			evaluation_model = null;
			evaluation_crossv = null;
			throw new QSARModelException(x.getMessage());
		}
		

	}
	/*
	public String getPredictedFieldName() {
		return predictedFieldName;
	}
	*/
	public double getDistance(Instance a) throws Exception {
        if (!filteredClassifier.getFilter().input(a)) {
            throw new Exception("Filter didn't make the test instance"
                  + " immediately available!");
          }
        filteredClassifier.getFilter().batchFinished();
        //Instance newInstance = filteredClassifier.getFilter().output();
        
        NearestNeighbourSearch nnSearch  = ((IBk)getClassifier()).getNearestNeighbourSearchAlgorithm();
        //Instances nn = nnSearch.kNearestNeighbours(newInstance, ((IBk)getClassifier()).getKNN());
          //System.out.println(nn);
          double[] distances = nnSearch.getDistances();
          double sum = 0;
          for (int i=0; i < distances.length; i++) sum += distances[i];
          return sum/distances.length;
    }
	public void processInstance(Subsets datasets, int subsetIndex, int objectIndex, Instances instances, Instance instance) throws Exception {
			  instance.setDataset(instances);
			  Classifier classifier = filteredClassifier;
			  if (filteredClassifier.getFilter() ==null)	classifier = getClassifier();
			  double clsLabel = classifier.classifyInstance(instance);
			  double[] d = classifier.distributionForInstance(instance);
			  IAtomContainersList data = datasets.get(subsetIndex).getData();
			  /*
			   * instances.classAttribute().value(instances.classAttribute().numValues()-1)
			   * instances.classAttribute().value(0)
			   */
			  if (data == null) return;
              NearestNeighbourSearch nnSearch  = ((IBk)getClassifier()).getNearestNeighbourSearchAlgorithm();
              
			  try {
				  boolean calculateDistance;
				  if (instances.classAttribute().isNominal()) {
					  if (modelStatistics == null) modelStatistics = createModelStatistics(true,datasets.toString());
					  data.setProperty(objectIndex, 
							  getTrainingPrefix() + "." +getSimilarityProperty() + "." + getClassificationProperty() , 
							  instances.classAttribute().value((int) clsLabel));
					  
					  for (int n=0; n < instances.classAttribute().numValues(); n++) {
						  data.setProperty(objectIndex,
								  getTrainingPrefix() + "." + getSimilarityProperty() + "." +								  
								  instances.classAttribute().value(n), new Double(d[n]));
						  
					  }
					  //histogram.update(new Float(d[(int)clsLabel]));
					  /*
                      Double dd = new Double(d[(int)clsLabel]);
                      data.setProperty(objectIndex, 
                    		  getTrainingPrefix() + "." + getSimilarityProperty() + "." + getClassificationProperty() + ".probability", 
                              dd);
                      */          
                      
                      calculateDistance = convertor.isEmptyClassProperty();
                      if (!calculateDistance)
                          modelStatistics.addValue(datasets.toString(),datasets.get(subsetIndex).toString(), 
                                  instances.classAttribute().value((int) clsLabel));
                  } else {
                	  if (modelStatistics == null) modelStatistics = createModelStatistics(false,datasets.toString());                	  
					  data.setProperty(objectIndex, 
							  getTrainingPrefix() + "." + getSimilarityProperty() + "." + getPredictedProperty(), 
							  new Float(clsLabel));
                      calculateDistance = true;
                  }
				  if (calculateDistance) {
                 		if (!filteredClassifier.getFilter().input(instance)) {
                            throw new Exception("Filter didn't make the test instance"
                                  + " immediately available!");
                          }
                        filteredClassifier.getFilter().batchFinished();					  
                      Instance newInstance = filteredClassifier.getFilter().output();
                      Instances nn = nnSearch.kNearestNeighbours(newInstance, ((IBk)getClassifier()).getKNN());
                      //System.out.println(nn);
                      double[] distances = nnSearch.getDistances();
                      double sum = 0;
                      for (int i=0; i < distances.length; i++) sum += distances[i];
                      Double dd = new Double(sum/distances.length);
                      data.setProperty(objectIndex, 
                    		  getTrainingPrefix() + "." + getSimilarityProperty(), 
                              dd);  
                      modelStatistics.addValue(datasets.toString(),datasets.get(subsetIndex).toString(), dd);                      
                      //histogram.update(dd);
                  } 		  
					
			  } catch (Exception x) {
				  x.printStackTrace();
				  data.setProperty(objectIndex, getTrainingPrefix() + "." + getSimilarityProperty(), x.getMessage());
			  }
	}
    /*
	public void classifyInstance(Instance instance) {
          System.out.print("Instance\t");
          System.out.println(instance);
          
          
            if (!filteredClassifier.getFilter().input(instance)) {
                throw new Exception("Filter didn't make the test instance"
                      + " immediately available!");
              }
            filteredClassifier.getFilter().batchFinished();
            
              Instance newInstance = filteredClassifier.getFilter().output();
  
          
          Instances nn = nnSearch.kNearestNeighbours(newInstance, ((IBk)getClassifier()).getKNN());
          System.out.println(nn);
          double[] distances = nnSearch.getDistances();
          double sum = 0;
          for (int i=0; i < distances.length; i++) sum += distances[i];
          data.setProperty(objectIndex, 
                  "distance", 
                  sum/distances.length);        
    
    }
*/    


	public Object process(Object object) throws AmbitException {
		// TODO Auto-generated method stub
		return null;
	}

	public void setClassifier(Classifier newClassifier) {
		filteredClassifier.setClassifier(newClassifier);
		evaluation_model = null;
		evaluation_crossv = null;
		setPredicting(false);
	}
	public Classifier getClassifier() {
		return filteredClassifier.getClassifier();
	}

    public synchronized Subsets2Instances getConvertor() {
        return convertor;
    }
    /*
    public void classifyInstance(Instance instance) {
    System.out.print("Instance\t");
    System.out.println(instance);
    
    
    if (!filteredClassifier.getFilter().input(instance)) {
    throw new Exception("Filter didn't make the test instance"
    + " immediately available!");
    }
    filteredClassifier.getFilter().batchFinished();
    
    Instance newInstance = filteredClassifier.getFilter().output();
    
    
    Instances nn = nnSearch.kNearestNeighbours(newInstance, ((IBk)getClassifier()).getKNN());
    System.out.println(nn);
    double[] distances = nnSearch.getDistances();
    double sum = 0;
    for (int i=0; i < distances.length; i++) sum += distances[i];
    data.setProperty(objectIndex, 
    "distance", 
    sum/distances.length);        
    
    }
    */
    @Override
    public String toString() {
        return getStatus(false);
    }
    public String getStatus(boolean verbose) {
        StringBuffer b = new StringBuffer();
        
        if (filteredClassifier != null) {
            IBk c = ((IBk)getClassifier());
            NearestNeighbourSearch nnSearch  = c.getNearestNeighbourSearchAlgorithm();
            DistanceFunction distance = nnSearch.getDistanceFunction();
            if (distance.getClass().getName().equals("weka.core.EuclideanDistance")) b.append("Euclidean distance");
            else b.append(distance.toString());
            
            if (verbose) {
                b.append(filteredClassifier.toString());
                b.append("\n\n");
            }
            b.append(", ");
            b.append(c.getKNN());
            if (verbose) b.append("Nearest Neighbours ");
            b.append("NN ");
        } else b.append("Distance "); 
        if (doesPrediction)
            b.append(" (Prediction)");
        else 
            b.append(" (Classification)");
        
        if (verbose) {
            
            b.append("\nDescriptors\n");
            
            ChemCastDescriptorsList d = getDescriptors();
            List slist = d.getList();
            for (int i=0; i < slist.size(); i++) {
                b.append(d.get(i));
                b.append("\n");
            }   
            b.append("\n");
            
            b.append(toString());
            if (evaluation_model!=null) {
                b.append("\nModel statistics:");
                b.append(evaluation_model.toSummaryString());
            }
            if (evaluation_crossv != null) {
                b.append("\nCrossvalidation statistics:");
                b.append(evaluation_crossv.toSummaryString());
            }
            
        }
        return b.toString();
    }

    public void setParameter(Object parameter, Object value) {
    	super.setParameter(parameter, value);
    	/*
        if (parameter.equals(ChemCastDescriptorsList.p_descriptors) && (value instanceof ChemCastDescriptorsList)) 
            setDescriptors((ChemCastDescriptorsList)value);
        else
        
        if (parameter.equals("Fingerprints")) {
            try {
                convertor.setFingerprints(Boolean.parseBoolean(value.toString()));
                setPredicting(false);
            } catch (Exception x) {
                convertor.setFingerprints(false);
            }
        } else
        */  
        if (parameter.equals("Distance")) {
            try {
                Object distanceFunct = Class.forName(value.toString()).newInstance();
                
                if (distanceFunct instanceof EuclideanDistance) {
                    ((IBk)filteredClassifier.getClassifier()).getNearestNeighbourSearchAlgorithm().setDistanceFunction((DistanceFunction)distanceFunct);
                    setPredicting(false);
                }    
            } catch (Exception x) {
                try {
                ((IBk)filteredClassifier.getClassifier()).getNearestNeighbourSearchAlgorithm().setDistanceFunction(new EuclideanDistance());
                setPredicting(false);
                } catch (Exception xx) {
                    x.printStackTrace();
                }
            }
        } else if (parameter.equals("kNN")) {
            int knn = 3;
            try {
                knn = Integer.parseInt(value.toString());
                try {
                    ((IBk)filteredClassifier.getClassifier()).setCrossValidate(false);
                    ((IBk)filteredClassifier.getClassifier()).setKNN(knn);
                    setPredicting(false);
                    } catch (Exception x) {
                        ((IBk)filteredClassifier.getClassifier()).setCrossValidate(true);
                        setPredicting(false);
                    }
            } catch (Exception x) {
                ((IBk)filteredClassifier.getClassifier()).setCrossValidate(true);
                setPredicting(false);
            }

        } else if (parameter.equals("crossvalidation")) {
            xval = false;
            try {
                xval = Boolean.parseBoolean(value.toString());
            } catch (Exception x) {
            	logger.error(x);
                xval = false;
            }
        }
        
    }
    public Object getParameter(Object parameter) {
        if (parameter.equals(ChemCastDescriptorsList.p_descriptors)) return getDescriptors();
        else
            return super.getParameter(parameter);
    }
    @Override
    public double predict(Object object1, Object object2) throws AmbitException {
    	try {
    		IAtomContainer c1 = null;
    		if (object1 instanceof IAtomContainer) c1 = (IAtomContainer) object1;
    		else throw new AmbitException(object1.getClass().getName() + " not an IAtomcontainer!");
    		IAtomContainer c2 = null;
    		if (object2 instanceof IAtomContainer) c2 = (IAtomContainer) object2;
    		else throw new AmbitException(object2.getClass().getName() + " not an IAtomcontainer!");
    		
    		if (pairwiseSimilarity==null) pairwiseSimilarity = convertor.createInstances("pairwise similarity");
	    	Instance instance1 = convertor.createInstance();
	    	instance1.setDataset(pairwiseSimilarity);
	    	convertor.property2Instance(c1,instance1,"?");
	    	Instance instance2 = convertor.createInstance();
	    	instance2.setDataset(pairwiseSimilarity);
	    	convertor.property2Instance(c2,instance2,"?");	    	
	        filteredClassifier.getFilter().input(instance1);
	        filteredClassifier.getFilter().input(instance2); 

	        filteredClassifier.getFilter().batchFinished();
	        Instance newInstance1 = filteredClassifier.getFilter().output();
	        
	        Instance newInstance2 = filteredClassifier.getFilter().output();
	
	        NearestNeighbourSearch nnSearch  = ((IBk)getClassifier()).getNearestNeighbourSearchAlgorithm();
	        DistanceFunction distance = nnSearch.getDistanceFunction();
            if (distance instanceof IDistanceFunction)
                return ((IDistanceFunction<Instance>) distance).getNativeComparison(newInstance1, newInstance2);
            else return (float) distance.distance(newInstance1, newInstance2);
	        
    	} catch (Exception x) {
    		throw new AmbitException(x);
    	}
        
    }
    
    public float predict(IAtomContainersList object1, int index1, IAtomContainersList object2, int index2) throws AmbitException {
    	try {
    		
    		if (pairwiseSimilarity==null) pairwiseSimilarity = convertor.createInstances("pairwise similarity");
	    	Instance instance1 = convertor.createInstance();
	    	instance1.setDataset(pairwiseSimilarity);
	    	convertor.property2Instance(object1,index1,instance1,"?");
	    	Instance instance2 = convertor.createInstance();
	    	instance2.setDataset(pairwiseSimilarity);
	    	convertor.property2Instance(object2,index2,instance2,"?");	    	
	        filteredClassifier.getFilter().input(instance1);
	        filteredClassifier.getFilter().input(instance2); 

	        filteredClassifier.getFilter().batchFinished();
	        Instance newInstance1 = filteredClassifier.getFilter().output();
	        
	        Instance newInstance2 = filteredClassifier.getFilter().output();
	
	        NearestNeighbourSearch nnSearch  = ((IBk)getClassifier()).getNearestNeighbourSearchAlgorithm();
	        DistanceFunction distance = nnSearch.getDistanceFunction();
            if (distance instanceof IDistanceFunction)
                return ((IDistanceFunction<Instance>) distance).getNativeComparison(newInstance1, newInstance2);
            else return (float) distance.distance(newInstance1, newInstance2);
	        
    	} catch (Exception x) {
    		throw new AmbitException(x);
    	}
        
    }

    @Override
    public IAmbitEditor getEditor() {
        
        return new WekaSimilarityEditor(this);
    }
    public String[] getProperties() {
    	if (doesPrediction)
	    	return new String[]{
	    	getTrainingPrefix() + "." + getSimilarityProperty(),
	    	getTrainingPrefix() + "." + getSimilarityProperty() + "." + getPredictedProperty()};
    	else
    		if (convertor.isEmptyClassProperty())
    		//if (classValues ==null)
    	    	return new String[]{
        	    	getTrainingPrefix() + "." + getSimilarityProperty()
        	    	};    		

    		else {
    			//String p = getTrainingPrefix() + "." + getSimilarityProperty() + "." + getClassificationProperty();
		    	String[] p =  new String[classValues.length+1];
		    	p[0] = getTrainingPrefix() + "." + getSimilarityProperty() + "." + getClassificationProperty();
	    	    for (int i=1; i < p.length;i++)  p[i] = getTrainingPrefix() + "." + getSimilarityProperty() + "." + classValues[i-1];
	    	    return p;
    		}
    }
    //InstancesSummaryPanel p = new InstancesSummaryPanel();
    /*
    PreprocessPanel p = new PreprocessPanel();
    p.setInstances(instances);
    JOptionPane.showMessageDialog(null,p);
    */
    /*
    JPanel p = new JPanel(new GridLayout(2,2));
    
    AttributeVisualizationPanel p1 = new AttributeVisualizationPanel();
    AttributeSummaryPanel p2 = new AttributeSummaryPanel();
    AttributeSelectionPanel p3 = new AttributeSelectionPanel();
    p.add(p1);p.add(p2);
    p.add(p3);
    p1.setInstances(instances);p2.setInstances(instances);p3.setInstances(instances);
    p1.setAttribute(0);p2.setAttribute(0);
    JOptionPane.showMessageDialog(null,p);
    
    ViewerDialog dlg = new ViewerDialog(null);
    dlg.setInstances(instances);
    dlg.showDialog();
      */          
    //double[] results = evaluation.evaluateModel(filteredClassifier, instances);
    /*
    instances = null;
    System.out.println(bestAttributes.getSearch().toString());
    double[][] ranks = ((Ranker)bestAttributes.getSearch()).rankedAttributes();
    System.out.println("Ranked attributes");
    for (int i=0; i < ranks.length;i++) {
        System.out.print(ranks[i][0]);
        System.out.print('\t');
        System.out.println(ranks[i][1]);
    }    
    
    System.out.println(bestAttributes.getEvaluator().toString());
    */
}

class WekaSimilarityEditor extends WekaSimilarityModelEditor {
    /**
     * 
     */
    private static final long serialVersionUID = -173937342874902993L;
    public WekaSimilarityEditor(WekaSimilarityModel model) {
        super(model);
        
    }
    @Override
    protected Object getObject() {
        return ((WekaSimilarityModel)model).filteredClassifier;
    }
    @Override
    protected Component createEditor() {

        return super.createEditor();
    }
 
}



