/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package toxTree.test.tree.cramer;

import junit.framework.TestCase;

import org.openscience.cdk.AtomContainer;
import org.openscience.cdk.interfaces.IAtomContainer;
import org.openscience.cdk.interfaces.ISetOfAtomContainers;

import toxTree.exceptions.DecisionMethodException;
import toxTree.exceptions.MolAnalyseException;
import toxTree.logging.TTLogger;
import toxTree.query.FunctionalGroups;
import toxTree.query.MolAnalyser;
import toxTree.query.MolFlags;
import toxTree.test.query.TestCasesFactory;
import toxTree.tree.cramer.RuleHasOnlySaltSulphonateSulphate;

/**
 * TODO add description
 * @author ThinClient
 * <b>Modified</b> 2005-9-25
 */
public class RuleHasOnlySaltSulphonateSulphateTest extends TestCase {

	public static void main(String[] args) {
		junit.textui.TestRunner
				.run(RuleHasOnlySaltSulphonateSulphateTest.class);
	}

	/*
	 * @see TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	/*
	 * @see TestCase#tearDown()
	 */
	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Constructor for RuleHasOnlySaltSulphonateSulphateTest.
	 * @param arg0
	 */
	public RuleHasOnlySaltSulphonateSulphateTest(String arg0) {
		super(arg0);
		TTLogger.configureLog4j(true);
	}
	public void testSalt() {
		RuleHasOnlySaltSulphonateSulphate rule = new RuleHasOnlySaltSulphonateSulphate();
		String[] answer = {"CCCCC(=O)[O-].[Na+]","[Ca+].O=C[O-].O=C[O-]"};
		
		for (int i=0; i < 2;i++) {
			IAtomContainer a = FunctionalGroups.createAtomContainer(answer[i]);
			
			try {
				MolAnalyser.analyse(a);
			} catch (MolAnalyseException x) {
				x.printStackTrace();
				fail();
			}
			try {
				assertTrue(rule.verifyRule(a));
				Object mf = a.getProperty(MolFlags.MOLFLAGS);
				assertNotNull(mf);
				ISetOfAtomContainers residues = ((MolFlags)mf).getResidues();
				assertNotNull(residues);
				assertEquals(1,residues.getAtomContainerCount());
			} catch (DecisionMethodException x) {
				x.printStackTrace();
				fail();			
			}
		}
	}
	public void testSulphateOfAmine() {
		RuleHasOnlySaltSulphonateSulphate rule = new RuleHasOnlySaltSulphonateSulphate();
		AtomContainer a = TestCasesFactory.phenazineMethosulphate();
		try {
			MolAnalyser.analyse(a);
		} catch (MolAnalyseException x) {
			x.printStackTrace();
			fail();
		}
		try {
			assertTrue(rule.verifyRule(a));
			Object mf = a.getProperty(MolFlags.MOLFLAGS);
			assertNotNull(mf);
			ISetOfAtomContainers residues = ((MolFlags)mf).getResidues();
			assertNotNull(residues);
			assertEquals(1,residues.getAtomContainerCount());
		} catch (DecisionMethodException x) {
			x.printStackTrace();
			fail();			
		}
	}	
	
	public void testSulphonate() {
		RuleHasOnlySaltSulphonateSulphate rule = new RuleHasOnlySaltSulphonateSulphate();
		IAtomContainer a = FunctionalGroups.createAtomContainer("[Na+].[O-]S(=O)(=O)CCCCS(=O)(=O)[O-].[Na+]");
		try {
			MolAnalyser.analyse(a);
		} catch (MolAnalyseException x) {
			x.printStackTrace();
			fail();
		}
		try {
			assertTrue(rule.verifyRule(a));
			Object mf = a.getProperty(MolFlags.MOLFLAGS);
			assertNotNull(mf);
			ISetOfAtomContainers residues = ((MolFlags)mf).getResidues();
			assertNotNull(residues);
			assertEquals(1,residues.getAtomContainerCount());
		} catch (DecisionMethodException x) {
			x.printStackTrace();
			fail();			
		}
	}
	public void testSulphate() {
		RuleHasOnlySaltSulphonateSulphate rule = new RuleHasOnlySaltSulphonateSulphate();
		IAtomContainer a = FunctionalGroups.createAtomContainer("[Na+].CCCCCCCCCCCCOCCOCCOCCOS([O-])(=O)=O");
		try {
			MolAnalyser.analyse(a);
		} catch (MolAnalyseException x) {
			x.printStackTrace();
			fail();
		}
		try {
			assertTrue(rule.verifyRule(a));
			Object mf = a.getProperty(MolFlags.MOLFLAGS);
			assertNotNull(mf);
			ISetOfAtomContainers residues = ((MolFlags)mf).getResidues();
			assertNotNull(residues);
			assertEquals(1,residues.getAtomContainerCount());
		} catch (DecisionMethodException x) {
			x.printStackTrace();
			fail();			
		}
	}			
	
	public void testHydroChloridOfAmine() {
		RuleHasOnlySaltSulphonateSulphate rule = new RuleHasOnlySaltSulphonateSulphate();
		IAtomContainer a = FunctionalGroups.createAtomContainer("[Cl-].[NH3+]C1CCCCC1");
		try {
			MolAnalyser.analyse(a);
		} catch (MolAnalyseException x) {
			x.printStackTrace();
			fail();
		}
		try {
			assertTrue(rule.verifyRule(a));
			Object mf = a.getProperty(MolFlags.MOLFLAGS);
			assertNotNull(mf);
			ISetOfAtomContainers residues = ((MolFlags)mf).getResidues();
			assertNotNull(residues);
			assertEquals(1,residues.getAtomContainerCount());
			assertTrue(FunctionalGroups.hasGroup(residues.getAtomContainer(0),
					FunctionalGroups.primaryAmine(false)));
		} catch (DecisionMethodException x) {
			x.printStackTrace();
			fail();			
		}
	}				
}
